using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

#region ___CODE_ADDEDE____ 
using XNACS1Lib;
#endregion

namespace SimpleXNA
{
    class Ball
    {
        private const float RADIUS = 0.5f;
        private const float SPEED = 30f / 120f;

        private Vector2 m_Velocity;
        private bool m_Expired;
        private int m_BounceCount;
        private XNACS1Circle m_Circle;

        public Ball(Vector2 p, int nth) 
        {
            m_Expired = false;
            m_BounceCount = 0;

            m_Velocity.X = XNACS1Base.RandomFloat(-0.5f, 0.5f);
            m_Velocity.Y = Math.Abs(m_Velocity.X) * 5.0f;
            m_Velocity.Normalize();
            m_Velocity *= (SPEED + (0.3f * XNACS1Base.RandomFloat() * SPEED));

            m_Circle = new XNACS1Circle(p, RADIUS);

            float min = 0.2f;
            float max = min + 0.5f;
            m_Circle.OutsideColor = Color.Red;
            m_Circle.CenterColor = new Color(new Vector3(
                                        XNACS1Base.RandomFloat(min, max),
                                        XNACS1Base.RandomFloat(min, max),
                                        XNACS1Base.RandomFloat(min, max)));

            m_Circle.Label = nth.ToString();
            m_Circle.LabelColor = Color.White;
        }
        public void Bounce() { m_BounceCount++; }
        public int NumBounce { get { return m_BounceCount; } }

        public Vector2 Velocity
        {
            set { m_Velocity = value; }
            get { return m_Velocity; }
        }

        public Vector2 Position
        {
            set { m_Circle.Center = value; }
            get { return m_Circle.Center; }
        }

        public float Radius { get { return RADIUS; } }

        public void Update()
        {
            m_Circle.Center += m_Velocity;
        
            BoundCollideStatus status = XNACS1Base.World.ClampAtWorldBound(m_Circle);

            switch (status)
            {
                case BoundCollideStatus.CollideBottom:
                    XNACS1Base.PlayACue("BallDie");
                    m_Expired = true;
                    break;
                case BoundCollideStatus.CollideTop:
                    m_Velocity.Y = -m_Velocity.Y;
                    XNACS1Base.PlayACue("Wall");
                    break;
                case BoundCollideStatus.CollideLeft:
                case BoundCollideStatus.CollideRight:
                    m_Velocity.X = -m_Velocity.X;
                    XNACS1Base.PlayACue("Wall");
                    break;
                case BoundCollideStatus.InsideBound:
                    break;
            }
        }

        public XNACS1Circle GetCircle
        {
            get { return m_Circle; }
        }

        public bool Expired
        {
            get
            {
                if (m_Expired)
                    m_Circle.RemoveFromAutoDrawSet();
                return m_Expired;
            }
        }

        public void DeleteBall()
        {
            m_Circle.RemoveFromAutoDrawSet();
        }

    }
}