using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

// Our own library
using XNACS1Lib;


namespace BlockBreaker_Program
{
    class Ball
    {
        private const float RADIUS = BlockBreaker.BLOCK_HEIGHT * 0.5f;
        private const float SPEED = BlockBreaker.UNIT_SPEED; // travels the width in 3 seconds
        private const float VToHRatio = 3.0f;

        private Vector2 m_Velocity;
        private bool m_Expired;
        private int m_BounceCount;

        private XNACS1Circle m_Circle;


        public Ball(Vector2 p, int nth)
        {
            m_Expired = false;
            m_BounceCount = 0;

            m_Velocity.X = XNACS1Base.RandomFloat(-0.5f, 0.5f);
            m_Velocity.Y = Math.Abs(m_Velocity.X) * 5.0f;
            m_Velocity.Normalize();
            m_Velocity *= (SPEED + (0.3f * XNACS1Base.RandomFloat() * SPEED));

            m_Circle = new XNACS1Circle(p, RADIUS);

            float min = 0.2f;
            float max = min + 0.5f;
            m_Circle.OutsideColor = Color.Red;
            m_Circle.CenterColor = new Color(new Vector3(
                                        XNACS1Base.RandomFloat(min, max),
                                        XNACS1Base.RandomFloat(min, max),
                                        XNACS1Base.RandomFloat(min, max)));
                                    
            m_Circle.Label = nth.ToString();
            m_Circle.LabelColor = Color.White;
        }

        public void Bounce() { m_BounceCount++; }
        public int NumBounce { get { return m_BounceCount; } }

        public XNACS1Circle GetCircle { get { return m_Circle; } }

        /// <summary>
        /// Make sure velocity's y component is at least VToHRatio that of X
        /// </summary>
        private void CheckVelocityRatio()
        {
            float ratio = Math.Abs(m_Velocity.Y) / Math.Abs(m_Velocity.X);
            if (ratio < VToHRatio)
            {
                float speed = m_Velocity.Length();
                float xComp = Math.Abs(m_Velocity.Y) / VToHRatio;
                if (m_Velocity.X < 0)
                    m_Velocity.X = -xComp;
                else
                    m_Velocity.X = xComp;
                m_Velocity.Normalize();
                m_Velocity *= speed;
            }
        }

        public Vector2 Velocity
        {
            set { m_Velocity = value; }
            get { return m_Velocity; }
        }

        public Vector2 Position
        {
            set { m_Circle.Center = value; }
            get { return m_Circle.Center; }
        }


        public float Radius { get { return RADIUS; } }

        public void Update()
        {
            m_Circle.Center += m_Velocity;
            CheckVelocityRatio();

            BoundCollideStatus status = XNACS1Base.World.ClampAtWorldBound(m_Circle);

            switch (status) {
                case BoundCollideStatus.CollideBottom:
                    m_Expired = true;
                    XNACS1Base.PlayACue("BallDie");
                    break;
                case BoundCollideStatus.CollideTop:
                    m_Velocity.Y = -m_Velocity.Y;
                    XNACS1Base.PlayACue("Wall");
                    break;
                case BoundCollideStatus.CollideLeft:
                    m_Velocity.X = -m_Velocity.X;
                    XNACS1Base.PlayACue("Wall");
                    break;
                case BoundCollideStatus.CollideRight:
                    m_Velocity.X = -m_Velocity.X;
                    XNACS1Base.PlayACue("Wall");
                    break;
                case BoundCollideStatus.InsideBound:
                    break;
            }
        }

        public bool Expired { 
            get {
                if (m_Expired)
                    m_Circle.RemoveFromAutoDrawSet();
                return m_Expired; 
            }
        }

    }
}