#region 1. Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

// Our own library
using XNACS1Lib;
#endregion

namespace SnakeAndLadder_Program
{
    /// <summary>
    ///  This is the main class. Notice this class must subclass from the XNACS1Base
    /// </summary>
    public class SnakeAndLadder : XNACS1Base
    {
        private const int TOTAL_CELLS = 200;
        private const float BORDER_SIZE = 0.4f;

        // Private application state
        // Location and size of the world
        private const float WORLD_MIN_X = -BORDER_SIZE;        
        private const float WORLD_MIN_Y = -BORDER_SIZE;
        private const float WORLD_WIDTH = 20.0f + (2*BORDER_SIZE);

        private GameBoardGUI m_Boardgui;
        private PlayStatus m_PlayStatus;

        private int m_NumMoves;
        private int m_TotalGold;
        private int m_TotalGoldLeft;

        private String m_Status;
        private String m_PlayerStatus;

#region override base class methods
        /// <summary>
        /// Initialize the application state
        /// </summary>
        protected override void InitializeWorld()
        {
            World.SetWorldCoordinate(new Vector2(WORLD_MIN_X, WORLD_MIN_Y), WORLD_WIDTH);
            World.SetBackgroundColor(Color.LightSkyBlue);

            // allocate memory for the application
            m_PlayStatus = new PlayStatus();

            m_Boardgui = new GameBoardGUI(TOTAL_CELLS, (int)(WORLD_WIDTH));
            m_PlayStatus.Initialize();

            m_NumMoves = 0;
            m_TotalGold = 0;
            m_TotalGoldLeft = m_Boardgui.TotalGoldOnBoard();
        }

        /// <summary>
        /// Update the application state. 
        /// </summary>
        protected override void UpdateWorld()
        {
            bool doneMoving = (!m_Boardgui.UpdatePlayStatus(m_PlayStatus));
            String goldMsg = null;
            if (m_PlayStatus.Gold > 0)
            {
                goldMsg = "Got Gold:" + m_PlayStatus.Gold;
                m_TotalGold += m_PlayStatus.Gold;
                m_TotalGoldLeft -= m_PlayStatus.Gold;
                m_PlayStatus.Gold = 0;
            }

            if (doneMoving)
            {
                m_PlayStatus.Initialize();
                if (XNACS1Base.GamePad.ButtonAClicked())
                {           
                    int move = 1 + RandomInt(5);
                    m_Boardgui.MoveBy(move, m_PlayStatus);
                    m_Status = "Rolling the die(" + move + ") ... ";
                    XNACS1Base.PlayACue( "HeroRun");
                    m_NumMoves++;
                 }

                 if (XNACS1Base.GamePad.ButtonBClicked())
                 {
                    m_Boardgui.ChanceIt(m_PlayStatus);
                    m_Status = "Chancing it ... ";
                    m_NumMoves++;
                 }
                 m_Status += m_PlayStatus.Status;
            }
            if (null != goldMsg)
                m_Status += " " + goldMsg;

            if (m_TotalGoldLeft > 0)
                m_PlayerStatus = "Number of moves so far=" + m_NumMoves + "  GoldLeft=" + m_TotalGoldLeft + "   Nuggets Collected=" + m_TotalGold;
            else
                m_PlayerStatus = "You have won! TotalMoves=" + m_NumMoves + " TotalGold=" + m_TotalGold;

            EchoToTopStatus(m_PlayerStatus);
            EchoToBottomStatus(m_Status);
            // Allows the default game to exit on Xbox 360 and Windows
            if (XNACS1Base.GamePad.Buttons.Back == ButtonState.Pressed)
                this.Exit();
        }

#endregion
    }
}
