using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

using XNACS1Lib;

namespace Othello_Program
{
    class GamePiece
    {
        private int m_Player;
        private XNACS1Circle m_Piece;

        private const float PIECE_SIZE = 0.48f;
        private static readonly Color[] PIECE_COLOR = null;

        static GamePiece() // this is a 'static initializer' - kind of like a constructor
        {                  // for the class.  If you don't understand this, ignore it & don't worry
                           // about it. :)
            PIECE_COLOR = new Color[2];
            PIECE_COLOR[PlayBoard.BlackPlayer] = Color.Black;
            PIECE_COLOR[PlayBoard.WhitePlayer] = Color.White;
        }

        // Constructor for individual instances of the GamePiece class
        public GamePiece(int x, int y, int p)
        {
            m_Player = p;
            Vector2 pos = new Vector2(x + 0.5f, y + 0.5f);
            m_Piece = new XNACS1Circle(pos, PIECE_SIZE);
            m_Piece.OutsideColor = m_Piece.CenterColor = PIECE_COLOR[p];
        }

        /// <summary>
        /// STUDENTS: This method will flip a given game piece.  If the piece is currently 
        /// owned by the BlackPlayer, then calling this method will cause the
        /// ownership to flip to the WhitePlayer.  Similar, if the piece is currently
        /// white, then calling FlipPiece will make it black
        /// </summary>
        public void FlipPiece()
        {
            if (m_Player == PlayBoard.WhitePlayer)
                m_Player = PlayBoard.BlackPlayer;
            else
                m_Player = PlayBoard.WhitePlayer;

            m_Piece.CenterColor = m_Piece.OutsideColor = PIECE_COLOR[m_Player];
        }

        /// <summary>
        /// STUDENTS: This method will flip a given game piece, so that the piece is now owned by
        /// the specified player (and the color will be adjusted so that the BlackPlayer's
        /// pieces are colored black, the WhitePlayer's pieces are colored white, etc)
        /// </summary>
        /// <param name="player">The number representing the player.  In the PlayToCell
        /// method, you should instead use BlackPlayer or WhitePlayer, instead of using
        /// 0 or 1 directly.  Failure to do so will result in the loss of points</param>
        public void FlipToPlayer(int player)
        {
            m_Player = player;
            m_Piece.CenterColor = m_Piece.OutsideColor = PIECE_COLOR[m_Player];
        }

        /// <summary>
        /// STUDENTS: This method will allow you (in PlayToCell, or HasValidPositions) to 
        /// determine if a given piece is owned by a specific player.  Note that this
        /// is different from figuring out if a given space is blank - see the 
        /// sample code in PlayToCell to see how to tell if a cell/square is empty.
        /// </summary>
        /// <param name="player">The number representing the player.  In the PlayToCell
        /// method, you should instead use BlackPlayer or WhitePlayer, instead of using
        /// 0 or 1 directly.  Failure to do so will result in the loss of points</param>
        /// <returns>true if this GamePiece object is owned by the specified
        /// player, false otherwise</returns>
        public bool IsPlayer(int player) { return (player == m_Player); }
    }
}