
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

// Our own library
using XNACS1Lib;

#endregion


// 
//
// ALL Media files must be PART of the solution (to be deployed over to the XBOX)!!


namespace BurstABubble_Program
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class BurstABubble : XNACS1Base
    {
        
        private const int MAX_THINGS_IN_FLIGHT = 3000;
        private const int MAX_ROCKS = 3000;
        private const int MAX_HEIGHT = 64;
        private const int MAX_WIDTH = 120;
        private const int MIN_BUBBLE_SIZE = 1;
        private const int MAX_BUBBLE_SIZE = 5;
        private const int MAX_FIREWORKS = 3;

        private PlayingSpace m_PlaySpace; 
        // private Player p;
        private Projectile[] m_thingsInFlight;
        private int m_thingsInFlight_NextUnoccupiedIndex = 0;
        private Bubble[] m_bubbles;
        private int m_bubbles_LastOccupiedIndex;

        private const float DELTA = 0.05f;
        private Vector2 m_CurrentVelocity;
        private XNACS1Rectangle m_VelocityIndicator;
        
        private int m_TotalRocks;
        private int m_TotalFireworks;

        // INSTRUCTOR VERSION (EXAMPLE SOLUTION) STARTS HERE:
        // We'll need a Random number generator:
        private Random m_Rand = new Random();
        // INSTRUCTOR VERSION (EXAMPLE SOLUTION) ENDS HERE:

        private const float WORLD_MIN_X = 0.0f;
        private const float WORLD_MIN_Y = 0.0f;
        private const float WORLD_WIDTH = 128.0f;

      
        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void InitializeWorld()
        {
            World.SetWorldCoordinate(new Vector2(WORLD_MIN_X, WORLD_MIN_Y), WORLD_WIDTH);
            // initialize graphics
            World.SetBackgroundTexture(@"bgTexture");

            // game state
            m_PlaySpace = new PlayingSpace(World.WorldMin, World.WorldMax);
            m_thingsInFlight = new Projectile[MAX_THINGS_IN_FLIGHT];

            m_bubbles = new Bubble[40];

            m_TotalRocks = 0;
            m_TotalFireworks = 0;

            m_VelocityIndicator = new XNACS1Rectangle();

            // initlaize games state
            Reset();
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void UpdateWorld()
        {
            // Allows the default game to exit on Xbox 360 and Windows
            if (XNACS1Base.GamePad.Buttons.Back == ButtonState.Pressed)
                this.Exit();

            if (XNACS1Base.GamePad.ButtonStartClicked())
                Reset();


            if (XNACS1Base.GamePad.ButtonAClicked())
            {
                m_TotalRocks++;
                Rock r = new Rock();
                r.GetUserInitInfo(m_CurrentVelocity);
                AddThingToFlightList(r);
                XNACS1Base.PlayACue( "Shooting");
            }

            if (XNACS1Base.GamePad.ButtonBClicked())
            {
                m_TotalFireworks++;
                Fireworks f = new Fireworks();
                f.GetUserInitInfo(m_CurrentVelocity);
                AddThingToFlightList(f);
                XNACS1Base.PlayACue( "Shooting");
            }

            Vector2 delta = XNACS1Base.GamePad.ThumbSticks.Left;
            m_CurrentVelocity += delta * DELTA;

            PlayRound();


            String msg = @"Press the 'A' button for a Rock, 'B' button for a Fireworks, and use the left thumbstick to control direction & speed!";
            XNACS1Base.EchoToTopStatus(msg);

            String amsg = @"Total Bubbles left=" + (m_bubbles_LastOccupiedIndex + 1) + "   Total # of Rocks Thrown=" + m_TotalRocks + "   Total # of Fireworks Launched=" + m_TotalFireworks;
            XNACS1Base.EchoToBottomStatus(amsg);

            //
            // Take care of velocity indicator
            m_VelocityIndicator.RemoveFromAutoDrawSet();
            Vector2 pa = new Vector2(0.1f, 0.1f);
            Vector2 pb = m_CurrentVelocity * 15.0f;
            m_VelocityIndicator = new XNACS1Rectangle(pa, pb, 1.0f, "");
            m_VelocityIndicator.OutsideColor = Color.Black;
            m_VelocityIndicator.CenterColor = Color.White;
        }

        public void PlayRound()
        {
            // Update anything in play from a previous round:
            // Could probably merge these two loops into one, single loop
            for (int i = 0; i < m_thingsInFlight_NextUnoccupiedIndex; i++)
            {
                // Move the Thing - returns true if thing was destroyed
                if (m_thingsInFlight[i].DoObjectUpdate() ||

                    // Has it left the playing field?
                    m_PlaySpace.IsOutOfBounds(m_thingsInFlight[i]))
                {
                    m_thingsInFlight[i].RemoveFromAutoDrawSet();
                    // remove it from play
                    for (int j = i; j < m_thingsInFlight_NextUnoccupiedIndex - 1; j++)
                    {
                        m_thingsInFlight[j] = m_thingsInFlight[j + 1];
                    }
                    // last slot now open
                    m_thingsInFlight_NextUnoccupiedIndex--;
                }
                else
                {
                    // only check for collisions if the projectile still exists
                    for (int iBubble = 0; iBubble <= m_bubbles_LastOccupiedIndex; iBubble++)
                    {
                        if (m_thingsInFlight[i].OverLapsOtherThing(m_bubbles[iBubble]))
                        {
                            m_bubbles[iBubble].RemoveFromAutoDrawSet();
                            // remove it from play
                            for (int j = iBubble; j < m_bubbles_LastOccupiedIndex; j++)
                            {
                                m_bubbles[j] = m_bubbles[j + 1];
                            }
                            m_bubbles_LastOccupiedIndex--;

                            // This looks weird, but is correct
                            // we need to 'back up' a spot, since
                            // we just 'deleted' the current bubble out from underneath
                            // the iBubble index.
                            iBubble--;
                            XNACS1Base.PlayACue( "BubbleBurst");
                        }
                    }
                }
            }
        }

        public bool AddThingToFlightList(Projectile pr)
        {
            if (m_thingsInFlight_NextUnoccupiedIndex >= m_thingsInFlight.Length)
            {
                return false;
            }

            m_thingsInFlight[m_thingsInFlight_NextUnoccupiedIndex] = pr;
            m_thingsInFlight_NextUnoccupiedIndex++;
            return true;
        }

        public void Reset()
        {
            m_CurrentVelocity.X = 0.5f;
            m_CurrentVelocity.Y = 0.5f;
            
            // STUDENT VERSION STARTS HERE:
            /* // We'll place a bunch of bubbles in a straight line.
            // How boring...
            float minX = MAX_BUBBLE_SIZE / 2;
            float maxX = MAX_WIDTH - (MAX_BUBBLE_SIZE / 2);
            float minY = MAX_BUBBLE_SIZE / 2;
            float maxY = MAX_HEIGHT - (MAX_BUBBLE_SIZE / 2);
            int howManyBubbles = m_bubbles.Length; // not needed, but
            // makes the code easier to read, below

            for (int i = 0; i < m_bubbles.Length; i++)
            {
                float x = ((float)(maxX - minX) / howManyBubbles)*i + minX;
                float y = ((float)(maxY - minY) / howManyBubbles)*i + minX;
                float radius = (float)(MIN_BUBBLE_SIZE + 1);
                Color c = Color.Magenta;

                // NOTE: It may be helpful to change the color of the bubbles,
                // so that you can track which one is going where...
                m_bubbles[i] = new Bubble(x, y, radius, c);
            }
            m_bubbles_LastOccupiedIndex = m_bubbles.Length - 1;
            // STUDENT VERSION ENDS HERE: */
            

            // INSTRUCTOR VERSION (EXAMPLE SOLUTION) STARTS HERE:
            for (int i = 0; i < m_bubbles.Length; i++)
            {
                float x;
                float y;
                float radius;
                Color c;

                // Students sometimes generate random numbers (with a decimal part)
                // like this:
                radius = (float)(m_Rand.Next(MIN_BUBBLE_SIZE, MAX_BUBBLE_SIZE)+m_Rand.NextDouble());

                if (i < 4) // 4 go 'close in' - between 30 & 50 units away
                {
                    x = (float)(m_Rand.NextDouble() * (50 - 30) + 30);
                    y = (float)(m_Rand.NextDouble() * (50 - 30) + 30);
                    c = Color.Red;
                }
                else if (i < 8) // 4 are placed 'farther away'
                {
                    x = (float)(m_Rand.NextDouble() * (90 - 60) + 60);
                    y = (float)(m_Rand.NextDouble() * (50 - 10) + 10);
                    c = Color.PowderBlue;
                }
                else // these can be placed randomly, or 
                {
                    // Randomly placed anywhere:
                    // x = (float)(m_Rand.NextDouble() * MAX_WIDTH);
                    // y = (float)(m_Rand.NextDouble() * MAX_HEIGHT);

                    // Not interesting, but it does make it easier to see the other,
                    // randomly placed circles:
                    x = 10;
                    y = i;
                    c = Color.Magenta;
                }

                m_bubbles[i] = new Bubble(x, y, radius, c);
            }
            m_bubbles_LastOccupiedIndex = m_bubbles.Length - 1;
            // INSTRUCTOR VERSION (EXAMPLE SOLUTION) ENDS HERE:
        }

    }
}
