using System;
using System.Text;
using Microsoft.Xna.Framework;

// If you haven't done so already, you should read the README.txt
// file, first


namespace BurstABubble_Program
{
    abstract public class Projectile : CircularThing
    {
        protected Vector2 m_Speed;

        // Default ctor does default initialization
        public Projectile()
        {
        }

        // Provided in case you want to put the projectile
        // somewhere, with a different radius
        public Projectile(float xPos, float yPos, float r) 
            : base(xPos, yPos, r)
        {
        }

        abstract public void GetUserInitInfo(Vector2 initV);

        virtual public bool DoObjectUpdate()
        {
            // This is a good example of what's call 'operator overloading'
            // In a nutshell, this line:
            //
            //  m_Position += m_Speed;
            //
            // Has the same effect as the following:
            //
            // m_Position.X += m_Speed.X;
            // m_Position.Y += m_Speed.Y;
            //
            // m_Position is actually a Vector2, which is similar to an
            // array with just 2 slots.  There's an entire class for this
            // because the concept of a vector (or an (x,y) position)
            // shows up frequently, and therefore it's a useful shorthand
            // to have a class that encapsulates common operations that are
            // specific to a vector (but that not all 2 element arrays would
            // want to implement)
            //
            // For example, Vector2 class implements 'operator overloading',
            // meaning that you can type in the following line:

            Center = Center + m_Speed;

            // and it will the two, individual parts of each vector together
            // (as explained in the comment above).

            // Texture is defined in XNACS1Circle class.
            Texture = GetTexture();

            return false;
        }

        virtual public void ApplyGravity()
        {
            m_Speed.Y -= 0.005f;
        }
    }
}
