using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

// Our own library
using XNACS1Lib;

namespace AlphabetHunt
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class CAlphabetHero
    {
        
        public const float kHeroHeight = 2.2f;
        
        private const float kGravity = 0.03f;

        private const float kHeroWidth = 0.8f;
        private const float kHalfHeroWidth = kHeroWidth * 0.5f;
        private const float kHalfHeroHeight = kHeroHeight * 0.5f;
        private const float kHeroMinYPos = CTreasureEngine.kFloorY + kHalfHeroHeight;

        
        private Vector2 m_HeroVelocity;

        private CSideScroll m_BackgrounScroll;
        private float m_MaxHeroHeight;

        private const String m_HeroLeft = @"HeroLeft";
        private const String m_HeroRight = @"HeroRight";
        
        private XNACS1Rectangle m_HeroRec;

        public CAlphabetHero(float worldWidth, float worldHeight, CSideScroll bg)
        {
            m_BackgrounScroll = bg;
            m_MaxHeroHeight = worldHeight - kHalfHeroHeight;

            m_HeroVelocity.X = 0.0f;
            m_HeroVelocity.Y = 0.0f;

            Vector2 pos = new Vector2(4.0f * kHeroWidth, kHeroMinYPos);
            m_HeroRec = new XNACS1Rectangle(pos, kHeroWidth, kHeroHeight, m_HeroRight);
        }

        public void UpdateHero()
        {
            m_HeroRec.Center += m_HeroVelocity;

            if (m_HeroRec.CenterY > kHeroMinYPos)
            {
                m_HeroVelocity.Y -= kGravity;
            }

            XNACS1Base.World.ClampAtWorldBound(m_HeroRec);

            if (m_HeroRec.CenterY < kHeroMinYPos)
            {
                m_HeroVelocity.Y = 0.0f;
                m_HeroRec.CenterY = kHeroMinYPos;
            }
        }

        public void SetHeroVelocity(Vector2 v)
        {
            const float scale = 0.3f;
            if (m_HeroRec.CenterY <= (kHeroMinYPos * 1.05f))
            {
                m_HeroVelocity.X = scale * v.X;
            }
            else
            {
                m_HeroVelocity.X = scale * 0.2f * v.X;
            }
            if (v.X != 0.0f)
            {
                if (v.X >= 0)
                    m_HeroRec.Texture = m_HeroRight;
                else
                    m_HeroRec.Texture = m_HeroLeft;
            }
        }

        public void HeroJump()
        {
            if (m_HeroRec.CenterY < 10.0f)
                m_HeroVelocity.Y = 0.5f;
        }
        public XNACS1Primitive GetPrimitive() 
        {
            return m_HeroRec;
        }

    }
}