#region 1. Using Statemetns
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

// Our own library
using XNACS1Lib;
#endregion

namespace SwitchAndMod_Program
{
    /// <summary>
    ///  This is the main class. Notice this class must subclass from the XNACS1Base
    /// </summary>
    public class SwitchAndMod : XNACS1Base
    {
        // Private application state
        // Location and size of the world
        private const float WORLD_MIN_X = 0.0f;          // lower left corner is (0, 0)
        private const float WORLD_MIN_Y = 0.0f;
        private const float WORLD_WIDTH = BackgroundStrip.STRIP_WIDTH * 17.0f;        // width of the world 

        BackgroundStrip m_Background;                    // defines and draws the background strip
        XNACS1Rectangle m_Cameleon;
        XNACS1Circle m_CameleonBase;


        #region 4. override base class methods
        /// <summary>
        /// Initialize the application state
        /// </summary>
        protected override void InitializeWorld()
        {
            World.SetWorldCoordinate(new Vector2(WORLD_MIN_X, WORLD_MIN_Y), WORLD_WIDTH);

            // Background must be created first!
            m_Background = new BackgroundStrip();

            // allocate memory for the application
            Vector2 pos = new Vector2(World.WorldMax.X * 0.5f, World.WorldMax.Y * 0.5f);
            m_CameleonBase = new XNACS1Circle(pos, 1.5f * BackgroundStrip.STRIP_WIDTH);
            m_CameleonBase.CenterColor = Color.White;

            float width = 1.5f * BackgroundStrip.STRIP_WIDTH;
            float height = 0.75f * width;
            m_Cameleon = new XNACS1Rectangle(pos, width, height, "l_chameleon");
        }

        /// <summary>
        /// Update the application state. 
        /// </summary>
        protected override void UpdateWorld()
        {
            float deltaL = 0f;
            float deltaR = 0f;
            if (XNACS1Base.GamePad.ThumbSticks.Left.X < 0f)
            {
                // deltaL is a positive number ("how far left?") so flip value
                deltaL = -1 * XNACS1Base.GamePad.ThumbSticks.Left.X * 0.1f * BackgroundStrip.STRIP_WIDTH;
            }
            if (XNACS1Base.GamePad.ThumbSticks.Right.X < 0f)
            {
                deltaL = -1 * XNACS1Base.GamePad.ThumbSticks.Right.X * 0.1f * BackgroundStrip.STRIP_WIDTH;
            }
            else if (XNACS1Base.GamePad.Dpad.Left == ButtonState.Pressed)
            {
                deltaL = 0.1f * BackgroundStrip.STRIP_WIDTH;
            }
            else if (XNACS1Base.GamePad.Triggers.Left > 0f)
            {
                deltaL = XNACS1Base.GamePad.Triggers.Left * 0.1f * BackgroundStrip.STRIP_WIDTH;
            }

            if (XNACS1Base.GamePad.ThumbSticks.Left.X > 0f)
            {
                deltaR = XNACS1Base.GamePad.ThumbSticks.Left.X * 0.1f * BackgroundStrip.STRIP_WIDTH;
            }
            if (XNACS1Base.GamePad.ThumbSticks.Right.X > 0f)
            {
                deltaR = XNACS1Base.GamePad.ThumbSticks.Right.X * 0.1f * BackgroundStrip.STRIP_WIDTH;
            }
            else if (XNACS1Base.GamePad.Dpad.Right == ButtonState.Pressed)
            {
                deltaR = 0.1f * BackgroundStrip.STRIP_WIDTH;
            }
            else if (XNACS1Base.GamePad.Triggers.Right > 0f)
            {
                deltaR = XNACS1Base.GamePad.Triggers.Right * 0.1f * BackgroundStrip.STRIP_WIDTH;
            }
            m_Cameleon.CenterX += deltaR - deltaL;
            World.ClampAtWorldBound(m_Cameleon);

            m_CameleonBase.Center = m_Cameleon.Center;

            if (deltaR > 0.0f)
            {
                m_Cameleon.Texture = "r_chameleon";
                XNACS1Base.PlayACue( "zoom");
            }
            else
                if (deltaL > 0.0f)
                {
                    m_Cameleon.Texture = "l_chameleon";
                    XNACS1Base.PlayACue( "zoom");
                }

            // Allows the default game to exit on Xbox 360 and Windows
            if (XNACS1Base.GamePad.Buttons.Back == ButtonState.Pressed)
                this.Exit();

            Color c = m_Background.GetColor(m_Cameleon.CenterX);
            m_CameleonBase.OutsideColor = c;

            EchoToTopStatus(@"Circle Position:" + m_Cameleon.Center + "   Center Color:" + c.ToString());
            EchoToBottomStatus(@"World Min:" + World.WorldMin + " and Max:" + World.WorldMax);
        }

        #endregion
    }
}
