#region 1. Using Statemetns
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

// Our own library
using XNACS1Lib;
#endregion

namespace SwitchAndMod_Guide_Program
{
    /// <summary>
    ///  This is the main class. Notice this class must subclass from the XNACS1Base
    /// </summary>
    public class SwitchAndMod_Guide : XNACS1Base
    {
        // Private application state
        // Location and size of the world
        private const float WORLD_MIN_X = 0.0f;          // lower left corner is (0, 0)
        private const float WORLD_MIN_Y = 0.0f;
        private const float WORLD_WIDTH = 20.0f;        // width of the world is 20.0

        XNACS1Circle m_ChameleonBase;                   // Circle around the Chameleon
        XNACS1Rectangle m_Chameleon;                    // The Chameleon
        
        float m_RecWidth;            // Width of the two constant color rectangles
        float m_RecHeight;           // Height of the two constant color rectangles

#region 4. override base class methods
        /// <summary>
        /// Initialize the application state
        /// </summary>
        protected override void InitializeWorld()
        {
            World.SetWorldCoordinate(new Vector2(WORLD_MIN_X, WORLD_MIN_Y), WORLD_WIDTH);
            
            m_RecHeight = World.WorldMax.Y - World.WorldMin.Y;    // rectangle covers the height
            m_RecWidth = WORLD_WIDTH/2.0f;            // width is half of area

            // A: The two constant color rectangles in the back ...
            Vector2 rCenter = new Vector2(m_RecWidth / 2.0f, m_RecHeight / 2.0f);
            XNACS1Rectangle r = new XNACS1Rectangle(rCenter, m_RecWidth, m_RecHeight);
            r.OutsideColor = Color.LightBlue;    // Left rectangle is in light blue
            r.CenterColor = Color.LightBlue;
            rCenter.X += m_RecWidth;       
            r = new XNACS1Rectangle(rCenter, m_RecWidth, m_RecHeight);
            r.OutsideColor = Color.Pink;         // Right rectangle is in pink
            r.CenterColor = Color.Pink;

            // B: Top-most object should be created last
            Vector2 chameleonPos = new Vector2(5.0f, 6.0f);
            m_ChameleonBase = new XNACS1Circle(chameleonPos, 2.0f);
            m_ChameleonBase.OutsideColor = Color.White;
            m_ChameleonBase.CenterColor = Color.White;
            m_Chameleon = new XNACS1Rectangle(chameleonPos, 3.0f, 3.0f, "l_chameleon");
                          // Look left "l_chameleon"
        }

        /// <summary>
        /// Update the application state. 
        /// </summary>
        protected override void UpdateWorld()
        {
            // Allows the default game to exit on Xbox 360 and Windows
            if (XNACS1Base.GamePad.ButtonBackClicked())
                this.Exit();

            // Label A: Pool the left/right triggers
            float deltaL = XNACS1Base.GamePad.Triggers.Left;
            float deltaR = XNACS1Base.GamePad.Triggers.Right;

            // Label B: Update Chameleon texture and audio
            if (deltaR > 0.0f)
            {
                m_Chameleon.Texture = "r_chameleon"; // right-facing texture
                XNACS1Base.PlayACue( "zoom");  // audio response 
            }
            else
                if (deltaL > 0.0f)
                {
                    m_Chameleon.Texture = "l_chameleon"; // left-facing texture
                    XNACS1Base.PlayACue( "zoom");  // audio response
                }

            // Label C: Update Chameleon position
            m_Chameleon.CenterX += deltaR - deltaL;      // chameleon position
            World.ClampAtWorldBound(m_Chameleon);              // clamp to window bounds
            m_ChameleonBase.Center = m_Chameleon.Center; // base circle position

            // Label D: Update Chameleon base circle color
            if (m_ChameleonBase.CenterX < m_RecWidth)
                m_ChameleonBase.CenterColor = Color.LightBlue; 
            else
                m_ChameleonBase.CenterColor = Color.Pink;

            // Label E: Echo status 
            EchoToTopStatus(@"ChameleonBase Position:" + m_ChameleonBase.Center + 
                             "   Center Color:" + m_ChameleonBase.CenterColor);
            EchoToBottomStatus(@"World Min:" + World.WorldMin + " and Max:" + World.WorldMax);
        }

#endregion
    }
}
