 #define INCLUDE_OPTIONAL_OUTPUT
using System;

namespace MonteCarloSolution
{
    class MonteCarloIntegrator
    {
        /// This solution doesn't require the students to know anything about instance
        /// variables...


        public void RunProgram()
        {
            Console.WriteLine("Welcome To The MonteCarlo Circle Area Estimator!!\n");
            
            // Key point: create the random object once, and reuse it.  not only does
            // this save memory (and puts less strain on the garbage collector), but since
            // the C# Random object (when created via the default constructor) seeds itself
            // with the current time (in milliseconds), which means that if you create
            // a new Random object each time through the loop (in Integrate), you won't be generating
            // random numbers - you'll generate a Random object, get it's first value (say, 10), then
            // create another Random object in the same millisecond, get it's first value (still 10), 
            // then create another Random object in the same millisecond, etc, etc.
            Random rng = new Random();

            // This array could be replaced by user I/O.  It's here so that
            // the programmer can try out several settings at once, and examine the accuracy, rather
            // than doing this all by hand.
            int[] howManyPoints = { 1, 10, 100, 500, 1000, 5000, 10000, 50000, 100000, 500000 };
            double[] error = new double[howManyPoints.Length];

            double maxX = 100;  // 'sheet of paper' is 100 units wide
            double maxY = 100;  // 'sheet of paper' is 100 units tall
            double radius = 10;
            double circleCenterX = 50;
            double circleCenterY = 50;

            // Since the students are writing this from scratch, they're
            // normally allowed to choose whatever values they want for the maxX/maxY/radius
            // variables.

            for (int i = 0; i < howManyPoints.Length; i++)
            {
                error[i] = Integrate(rng, maxX, maxY, circleCenterX, circleCenterY, radius, howManyPoints[i]);
            }

            Console.WriteLine("Max X:            {0}", maxX);
            Console.WriteLine("Max Y:            {0}", maxY);
            Console.WriteLine("Radius of Circle: {0}", radius);
            Console.WriteLine("Num Points      Off by (%)");

            for (int i = 0; i < howManyPoints.Length; i++)
                Console.WriteLine("{0}\t\t{1}", howManyPoints[i], error[i]);

            Console.WriteLine("Type <enter> to quit");
            String ans = Console.ReadLine();
        }

        private double Integrate(Random rng,  double maxX, double maxY, double circleCenterX, double circleCenterY, double circleRadius, int howManyPoints)
        {
#if INCLUDE_OPTIONAL_OUTPUT
            Console.WriteLine("Max X:{0}", maxX);
            Console.WriteLine("Max Y:{0}", maxY);
            Console.WriteLine("Total Area of Surrounding Space: {0}", maxX *maxY );
            Console.WriteLine("Basing our estimate on {0} randomly selected points", howManyPoints);
#endif

            // NOTE: If students know how to create classes, the paper & the circle (composted of
            // a Point class and a radius (double) works great :)
            double x;
            double y;

            int cInside = 0; // count of the number of 'darts' that randomly landed inside the circles

            for (int i = 0; i < howManyPoints; i++)
            {
                getRandomPoint(rng, maxX, out x, maxY, out y);
                if (pointIsInsideCircle(x, y, circleCenterX, circleCenterY, circleRadius ))
                    cInside++;
            }

            return calculateError(howManyPoints, cInside, maxX, maxY, circleRadius); ;
        }

        private void getRandomPoint(Random rng, double maxX, out double x, double maxY, out double y)
        {
            x = rng.NextDouble() * maxX;
            y = rng.NextDouble() * maxY;
        }

        private bool pointIsInsideCircle(double ptX, double ptY, double circX, double circY, double circR)
        {
            double dist = Math.Sqrt(Math.Pow(circX - ptX, 2) + Math.Pow(circY - ptY, 2));
            return dist <= circR;
        }

        private double calculateError(int total, int cInside, double maxX, double maxY, double circR )
        {
            double actualArea = Math.PI * circR * circR;

            double estArea = maxX * maxY * cInside / total; // Accidentally doing 
            // integer division is a common student error, here

            double offBy = estArea - actualArea;

            double error = offBy / actualArea * 100;

#if INCLUDE_OPTIONAL_OUTPUT
            Console.WriteLine("% of Randomly Picked Points Inside The Circle: {0}", cInside);
            Console.WriteLine("Estimated Total Area Of the Circle: {0}", estArea);
            Console.WriteLine("Actual Total Area Of the Circle: {0}", actualArea);
            Console.WriteLine("The estimate was off by {0}, which is {1}% of the real circle", offBy, error);
            Console.WriteLine("************************************************************");
#endif
            return  error;
        }
    }
}
