// #define STUDENT_VERSION
#define INSTRUCTOR_VERSION

#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

// Our own library
using XNACS1Lib;

#endregion


// 
//
// ALL Media files must be PART of the solution (to be deployed over to the XBOX)!!


namespace InsectsGarden_Program
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class InsectsGarden : XNACS1Base
    {
        private const float WorldWidth = 40.0f;
        private const int INSECT_TIMER = 100;
        private const int INSECT_TIMER_BASE = 100;
        private const int NUM_FLOWERS = 150;

        private const float NET_POSY = 1.8f;

        private XNACS1Rectangle m_Hero;

        private InsectCollection m_Insects;
        private Insect m_Insect;
        private int m_InsectTimer;
        private int m_LastScore;


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void InitializeWorld()
        {
            World.SetWorldCoordinate(new Vector2(-1.0f, -1.0f), WorldWidth);
            XNACS1Base.World.SetBackgroundColor(Color.LawnGreen);

            // Create the folowers
            for (int i = 0; i < NUM_FLOWERS; i++)
            {
                float x = RandomFloat(GardenWidth);
                float y = 0.5f + RandomFloat(GardenHeight - 1.5f);
                XNACS1Circle c = new XNACS1Circle(new Vector2(x, y), 0.5f);
                c.Texture = "flower" + RandomInt(6);
            }

            m_Insects = new InsectCollection(this);

            m_Insect = null;
            m_InsectTimer = 0;
            m_LastScore = 0;

            m_Hero = new XNACS1Rectangle(new Vector2(World.WorldMax.X / 2.0f, World.WorldMax.Y / 2.0f), 4.0f, 4.0f, "lnet");
        }

        /// <summary>
        /// Update application state and draw the current state of all game pieces
        /// </summary>
        protected override void UpdateWorld()
        {
            // Update state

            // Allows the default game to exit on Xbox 360 and Windows
            if (XNACS1Base.GamePad.Buttons.Back == ButtonState.Pressed)
                this.Exit();


            Vector2 delta = XNACS1Base.GamePad.ThumbSticks.Left;
            bool once = false;

            if (delta.X > 0.0f)
            {
                m_Hero.Texture = "rnet";
                once = true;
            }
            else if (delta.X < 0.0f)
            {
                m_Hero.Texture = "lnet";
                once = true;
            }

            if (once)
            {
                delta.Normalize();
            }
            m_Hero.Center = m_Hero.Center + (delta * 0.5f);
            World.ClampAtWorldBound(m_Hero);

            if (m_InsectTimer > 0)
            {
                m_InsectTimer--;
                float x = RandomFloat(-0.2f, 0.2f);
                float y = RandomFloat(-0.2f, 0.2f);
                m_Insect.UpdatePosition(x, y);

                Vector2 pos = m_Insect.Position;
                if (m_Insects.CatchInsect(pos.X, pos.Y, m_Hero.CenterX, m_Hero.CenterY + NET_POSY, m_Insect.Score()))
                {
                    m_LastScore = m_Insect.Score();
                    m_Insect.KillInsect();
                    m_Insect = null;
                    m_InsectTimer = -1;
                    XNACS1Base.PlayACue( "netted");
                }
            }

            if (m_InsectTimer <= 0)
            {
                if (m_InsectTimer == 0)
                    m_LastScore = 0;
                if (null != m_Insect)
                    m_Insect.KillInsect();
                m_Insect = null;
                if (RandomFloat() > 0.35f)
                {
                    m_Insects.PlaceANewInsect();
                    XNACS1Base.PlayACue( "squirt");
                }
            }

            XNACS1Base.EchoToBottomStatus(m_Insects.InsectsStatus());

            String msg;
            if (m_LastScore == 0)
                msg = "Missed This one!";
            else
                msg = "Caught! Score=" + m_LastScore;

            if (m_InsectTimer > 0)
                XNACS1Base.EchoToTopStatus(msg + "  " + "New Insect will be gone in: " + m_InsectTimer);
            else
                XNACS1Base.EchoToTopStatus("No insect spotted");

        }


        ///////////////////////////////////////////////////////////////////////
        // The following methods will be used by the students' InsectCollectionCenter class...
        ///////////////////////////////////////////////////////////////////////
        public float GardenWidth { get { return World.WorldMax.X - 1.0f; } }
        public float GardenHeight { get { return World.WorldMax.Y - 1.0f; } }

        public void CreateInsectAt(double x, double y)
        {
            m_Insect = new Insect((float)x, (float)y);
            m_InsectTimer = INSECT_TIMER_BASE + RandomInt(INSECT_TIMER);
        }
    }
}
