#define INSTRUCTOR_VERSION
//#define STUDENT_VERSION

using System;

namespace StarterFiles
{
    /// <summary>
    /// This class serves as the 'User Interface', or UI, to 
    /// the PeriodicTable class.
    /// </summary>
    class PeriodicTableUI
    {
        PeriodicTable m_theTable;
        private const int QUIT = 0;
        private const int PRINTALL = 1;
        private const int PRINTONE = 2;
        private const int ADDELEMENT = 3;
        private const int EDITELEMENT = 4;

        /// <summary>
        /// The constructor will create a single instance
        /// of the PeriodicTable class
        /// </summary>
        public PeriodicTableUI()
        {
            m_theTable = new PeriodicTable();
        }

        public void RunProgram()
        {
            int userChoice;

            Console.WriteLine("Welcome to the Periodic Table Program!\n\n");

            do
            {
                Console.WriteLine("\nPlease select from the following menu of options:");
                Console.WriteLine("0) Quit the program");
                Console.WriteLine("1) Print all the elements");
                Console.WriteLine("2) Print out a particular element");
                Console.WriteLine("3) Add a new element");
                Console.WriteLine("4) Edit an existing element");

                userChoice = getValidUserInput(0, 4);
                switch (userChoice)
                {
                    case PRINTALL:
                        printAllElements();
                        break;

                    case PRINTONE:
                        printOneElement();
                        break;

                    case ADDELEMENT:
                        addElementGetInput();
                        break;

                    case EDITELEMENT:
                        editElement();
                        break;

                    case QUIT:
                        // empty, but prevents us from falling into the 'default' case
                        break;
                    default:
                        Console.WriteLine("ERROR: You've selected an option which isn't supported!");
                        break;
                }
            }
            while (userChoice != QUIT);

            Console.WriteLine("\nThank you for using the Periodic Table Program!");
            Console.WriteLine("(Please press the <Enter> key to exit)");
            Console.ReadLine();
        }

        private void printAllElements()
        {
            ///////////////////////////////////////////////////////////////////////
            // FOR STUDENT STARTER PROJECT: REMOVE CODE BELOW, STARTING HERE     //
#if INSTRUCTOR_VERSION/////////////////////////////////////////////////////////////

            for (int i = 1; i <= m_theTable.MaxAtomicNumber; i++)
            {
                if (m_theTable.ElementHasData(i))
                {
                    m_theTable.getElement(i).Print();
                }
                else
                {
                    Console.WriteLine("Atomic #{0}: <Missing>", i);
                }
            }

#endif      ///////////////////////////////////////////////////////////////////////
            // FOR STUDENT STARTER PROJECT: REMOVE ABOVE CODE, ENDING HERE       //
            ///////////////////////////////////////////////////////////////////////

            ///////////////////////////////////////////////////////////////////////
#if STUDENT_VERSION////////////////////////////////////////////////////////////////
            // STUDENTS: YOU NEED TO WRITE YOUR CODE BELOW:


            // STUDENTS: YOU NEED TO WRITE YOUR CODE ABOVE:
#endif      ///////////////////////////////////////////////////////////////////////
            ///////////////////////////////////////////////////////////////////////

            // STUDENTS: IMPLEMENT THIS!!
        }

        private void printOneElement()
        {
            Console.WriteLine("What is the atomic number of the element you would like to print?");
            Console.WriteLine("Valid atomic numbers range from 1 to {0}", m_theTable.MaxAtomicNumber);
            int atomicNum = getValidUserInput(1, m_theTable.MaxAtomicNumber);

            // STUDENTS: IMPLEMENT THIS!!

            ///////////////////////////////////////////////////////////////////////
            // FOR STUDENT STARTER PROJECT: REMOVE CODE BELOW, STARTING HERE     //
#if INSTRUCTOR_VERSION/////////////////////////////////////////////////////////////

            if (m_theTable.ElementHasData(atomicNum))
                m_theTable.getElement(atomicNum).Print();
            else
            {
                addElementIfUserWantsTo(atomicNum);
            }

#endif      ///////////////////////////////////////////////////////////////////////
            // FOR STUDENT STARTER PROJECT: REMOVE ABOVE CODE, ENDING HERE       //
            ///////////////////////////////////////////////////////////////////////

            ///////////////////////////////////////////////////////////////////////
#if STUDENT_VERSION////////////////////////////////////////////////////////////////
            // STUDENTS: YOU NEED TO WRITE YOUR CODE BELOW:


            // STUDENTS: YOU NEED TO WRITE YOUR CODE ABOVE:
#endif      ///////////////////////////////////////////////////////////////////////
            ///////////////////////////////////////////////////////////////////////

        }

        // These are provided for your use
        private void addElementGetInput()
        {
            Console.WriteLine("What is the atomic number of the element you would like to add?");
            Console.WriteLine("Note that you can't replace an existing element");
            Console.WriteLine("Valid atomic numbers range from 1 to {0}", m_theTable.MaxAtomicNumber);
            int atomicNum = getValidUserInput(1, m_theTable.MaxAtomicNumber);
            addElement(atomicNum);
        }

        private void addElementIfUserWantsTo(int atomicNum)
        {
            Console.WriteLine("There is no data for that element!");
            Console.WriteLine("Would you like to add it now? (type 'y' or 'Y' to do so!)");
            char letter = (char)Console.Read();
            Console.ReadLine(); // clear input buffer
            if ('y' == letter || 'Y' == letter)
                addElement(atomicNum);
        }

        private void addElement(int atomicNum)
        {

            // STUDENTS: IMPLEMENT THIS!!

            ///////////////////////////////////////////////////////////////////////
            // FOR STUDENT STARTER PROJECT: REMOVE CODE BELOW, STARTING HERE     //
#if INSTRUCTOR_VERSION/////////////////////////////////////////////////////////////

            if (m_theTable.ElementHasData(atomicNum))
            {
                Console.WriteLine("That element already has data (and you're not allowed to overwrite it!)");
                m_theTable.getElement(atomicNum).Print();
            }
            else
            {
                Console.WriteLine("There is no data for that element, so you can add your own");
                Console.WriteLine("What is the element's name?");
                string name = Console.ReadLine();
                Console.WriteLine("What is the element's symbol?");
                string sym = Console.ReadLine();

                Console.WriteLine("What is the element's atomic weight?");
                double weight = getValidUserInputDouble(0, Double.MaxValue);

                Element e = new Element(atomicNum, name, sym, weight);
                m_theTable.setElement(atomicNum, e);

                Console.WriteLine("Element created!");
            }

#endif      ///////////////////////////////////////////////////////////////////////
            // FOR STUDENT STARTER PROJECT: REMOVE ABOVE CODE, ENDING HERE       //
            ///////////////////////////////////////////////////////////////////////

            ///////////////////////////////////////////////////////////////////////
#if STUDENT_VERSION////////////////////////////////////////////////////////////////
            // STUDENTS: YOU NEED TO WRITE YOUR CODE BELOW:


            // STUDENTS: YOU NEED TO WRITE YOUR CODE ABOVE:
#endif      ///////////////////////////////////////////////////////////////////////
            ///////////////////////////////////////////////////////////////////////

        }

        private void editElement()
        {
            Console.WriteLine("What is the atomic number of the element you would like to edit?");
            Console.WriteLine("Valid atomic numbers range from 1 to {0}", m_theTable.MaxAtomicNumber);
            int atomicNum = getValidUserInput(1, m_theTable.MaxAtomicNumber);

            // STUDENTS: IMPLEMENT THIS!!
            
            ///////////////////////////////////////////////////////////////////////
            // FOR STUDENT STARTER PROJECT: REMOVE CODE BELOW, STARTING HERE     //
#if INSTRUCTOR_VERSION/////////////////////////////////////////////////////////////

            if (!m_theTable.ElementHasData(atomicNum)) // this means that
            {   // the index is ok, BUT there's no object there...
                addElementIfUserWantsTo(atomicNum);
            }
            else
            {
                Element e = m_theTable.getElement(atomicNum);

                Console.WriteLine("Here is the element's current info:");
                e.Print();
                Console.WriteLine("What would you like the new name to be? (press enter on blank line to remain unchanged)");
                string name = Console.ReadLine();
                if ("" != name)
                {
                    e.setName(name);
                }

                Console.WriteLine("What would you like the new symbol to be? (press enter on blank line to remain unchanged)");
                string sym = Console.ReadLine();
                if ("" != sym)
                {
                    e.setSymbol(sym);
                }

                Console.WriteLine("What would you like the new weight to be?");
                double weight = getValidUserInputDouble(0, Double.MaxValue);
                e.setWeight(weight);

                Console.WriteLine("Here is the element's new info:");
                e.Print();

                // note that since "e" is a reference to the same object
                // that the periodic table refers to, we don't need to 
                // update the table object in any way...
            }
#endif      ///////////////////////////////////////////////////////////////////////
            // FOR STUDENT STARTER PROJECT: REMOVE ABOVE CODE, ENDING HERE       //
            ///////////////////////////////////////////////////////////////////////

            ///////////////////////////////////////////////////////////////////////
#if STUDENT_VERSION////////////////////////////////////////////////////////////////
            // STUDENTS: YOU NEED TO WRITE YOUR CODE BELOW:


            // STUDENTS: YOU NEED TO WRITE YOUR CODE ABOVE:
#endif      ///////////////////////////////////////////////////////////////////////
            ///////////////////////////////////////////////////////////////////////

        }

        private double getValidUserInputDouble(double min, double max)
        {
            double userChoice;
            bool notANum;
            while ((notANum = !Double.TryParse(Console.ReadLine(), out userChoice)) ||
                userChoice < min ||
                userChoice > max)
            {
                if (notANum)
                    Console.WriteLine("You need to type in a decimal number, no less than {0}, and no greater than {1}", min, max);
                else if (userChoice < min)
                    Console.WriteLine("You chose {0}, which is less than the smallest allowed number ({1})", userChoice, min);
                else if (userChoice > max)
                    Console.WriteLine("You chose {0}, which is larger than the largest allowed number ({1})", userChoice, max);
            }

            return userChoice;
        }

        // A perfect place for generics.... which we're avoiding, since
        // we haven't covered them yet :)
        private int getValidUserInput(int min, int max)
        {
            int userChoice;
            bool notANum;
            while ((notANum = !Int32.TryParse(Console.ReadLine(), out userChoice)) ||
                userChoice < min ||
                userChoice > max)
            {
                if (notANum)
                    Console.WriteLine("You need to type in a whole number, no less than {0}, and no greater than {1}", min, max);
                else if (userChoice < min)
                    Console.WriteLine("You chose {0}, which is less than the smallest allowed number ({1})", userChoice, min);
                else if (userChoice > max)
                    Console.WriteLine("You chose {0}, which is larger than the largest allowed number ({1})", userChoice, max);
            }

            return userChoice;
        }
    }
}