using System;

namespace StarterFiles
{
    /// <summary>
    /// This class serves as the 'User Interface', or UI, to 
    /// the PeriodicTable class.
    /// </summary>
    class PeriodicTableUI
    {
        PeriodicTable m_theTable;
        private const int QUIT = 0;
        private const int PRINTALL = 1;
        private const int PRINTONE = 2;
        private const int ADDELEMENT = 3;
        private const int EDITELEMENT = 4;

        /// <summary>
        /// The constructor will create a single instance
        /// of the PeriodicTable class
        /// </summary>
        public PeriodicTableUI()
        {
            m_theTable = new PeriodicTable();
        }

        public void RunProgram()
        {
            int userChoice;

            Console.WriteLine("Welcome to the Periodic Table Program!\n\n");

            do
            {
                Console.WriteLine("\nPlease select from the following menu of options:");
                Console.WriteLine("0) Quit the program");
                Console.WriteLine("1) Print all the elements");
                Console.WriteLine("2) Print out a particular element");
                Console.WriteLine("3) Add a new element");
                Console.WriteLine("4) Edit an existing element");

                userChoice = getValidUserInput(0, 4);
                switch (userChoice)
                {
                    case PRINTALL:
                        printAllElements();
                        break;

                    case PRINTONE:
                        printOneElement();
                        break;

                    case ADDELEMENT:
                        addElementGetInput();
                        break;

                    case EDITELEMENT:
                        editElement();
                        break;

                    case QUIT:
                        // empty, but prevents us from falling into the 'default' case
                        break;
                    default:
                        Console.WriteLine("ERROR: You've selected an option which isn't supported!");
                        break;
                }
            }
            while (userChoice != QUIT);

            Console.WriteLine("\nThank you for using the Periodic Table Program!");
            Console.WriteLine("(Please press the <Enter> key to exit)");
            Console.ReadLine();
        }

        private void printAllElements()
        {
            // STUDENTS: IMPLEMENT THIS!!
        }

        private void printOneElement()
        {
            Console.WriteLine("What is the atomic number of the element you would like to print?");
            Console.WriteLine("Valid atomic numbers range from 1 to {0}", m_theTable.MaxAtomicNumber);
            int atomicNum = getValidUserInput(1, m_theTable.MaxAtomicNumber);

            // STUDENTS: IMPLEMENT THIS!!
        }

        // These are provided for your use
        private void addElementGetInput()
        {
            Console.WriteLine("What is the atomic number of the element you would like to add?");
            Console.WriteLine("Note that you can't replace an existing element");
            Console.WriteLine("Valid atomic numbers range from 1 to {0}", m_theTable.MaxAtomicNumber);
            int atomicNum = getValidUserInput(1, m_theTable.MaxAtomicNumber);
            addElement(atomicNum);
        }

        private void addElementIfUserWantsTo(int atomicNum)
        {
            Console.WriteLine("There is no data for that element!");
            Console.WriteLine("Would you like to add it now? (type 'y' or 'Y' to do so!)");
            char letter = (char)Console.Read();
            Console.ReadLine(); // clear input buffer
            if ('y' == letter || 'Y' == letter)
                addElement(atomicNum);
        }

        private void addElement(int atomicNum)
        {
            // STUDENTS: IMPLEMENT THIS!!
        }

        private void editElement()
        {
            Console.WriteLine("What is the atomic number of the element you would like to edit?");
            Console.WriteLine("Valid atomic numbers range from 1 to {0}", m_theTable.MaxAtomicNumber);
            int atomicNum = getValidUserInput(1, m_theTable.MaxAtomicNumber);

            // STUDENTS: IMPLEMENT THIS!!
        }

        private double getValidUserInputDouble(double min, double max)
        {
            double userChoice;
            bool notANum;
            while ((notANum = !Double.TryParse(Console.ReadLine(), out userChoice)) ||
                userChoice < min ||
                userChoice > max)
            {
                if (notANum)
                    Console.WriteLine("You need to type in a decimal number, no less than {0}, and no greater than {1}", min, max);
                else if (userChoice < min)
                    Console.WriteLine("You chose {0}, which is less than the smallest allowed number ({1})", userChoice, min);
                else if (userChoice > max)
                    Console.WriteLine("You chose {0}, which is larger than the largest allowed number ({1})", userChoice, max);
            }

            return userChoice;
        }

        // A perfect place for generics.... which we're avoiding, since
        // we haven't covered them yet :)
        private int getValidUserInput(int min, int max)
        {
            int userChoice;
            bool notANum;
            while ((notANum = !Int32.TryParse(Console.ReadLine(), out userChoice)) ||
                userChoice < min ||
                userChoice > max)
            {
                if (notANum)
                    Console.WriteLine("You need to type in a whole number, no less than {0}, and no greater than {1}", min, max);
                else if (userChoice < min)
                    Console.WriteLine("You chose {0}, which is less than the smallest allowed number ({1})", userChoice, min);
                else if (userChoice > max)
                    Console.WriteLine("You chose {0}, which is larger than the largest allowed number ({1})", userChoice, max);
            }

            return userChoice;
        }
    }
}