#region 1. Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

// Our own library
using XNACS1Lib;
#endregion

namespace SnakeAndLadder_Program
{
    /// <summary>
    ///  This is the main class. Notice this class must subclass from the XNACS1Base
    /// </summary>
    public class SnakeAndLadder : XNACS1Base
    {
        private const int TOTAL_CELLS = 50;             // number of cells (row x column)
        private const float BORDER_SIZE = 0.4f;        //  boundaries around game board

        // Location and size of the world
        private const float WORLD_MIN_X = -BORDER_SIZE;   // lower-left corner of application
        private const float WORLD_MIN_Y = -BORDER_SIZE;   //   window is (-0.4, -0.4)
        private const float WORLD_WIDTH = 10.0f + (2*BORDER_SIZE); // compensate for boundaries

        private GameBoardGUI m_Boardgui;                  // the game board
        private String m_Status;


#region override base class methods
        /// <summary>
        /// Initialize the application state
        /// </summary>
        protected override void InitializeWorld()
        {
            World.SetWorldCoordinate(new Vector2(WORLD_MIN_X, WORLD_MIN_Y), WORLD_WIDTH);
            World.SetBackgroundColor(Color.LightSkyBlue);                           // background color

            m_Boardgui = new GameBoardGUI(TOTAL_CELLS, (int)(WORLD_WIDTH)); // memory for the Gameboard
        }

        /// <summary>
        /// Update the application state. 
        /// </summary>
        protected override void UpdateWorld()
        {
           if (XNACS1Base.GamePad.ButtonAClicked())  {       // A-Button clicked
              int move = 1 + RandomInt(5);                   // Random between 1 to 6
              m_Boardgui.MoveBy(move);                              // invoke game board
              m_Status = @"Button A clicked: move by " + move;      // echo status
              XNACS1Base.PlayACue( "HeroRun");
           }
                     
           // Allows the default game to exit on Xbox 360 and Windows
           if (XNACS1Base.GamePad.Buttons.Back == ButtonState.Pressed)
                this.Exit();

            EchoToBottomStatus(m_Status);       // echo status
        }
#endregion
    }
}
