#define INSTRUCTOR_VERSION
//#define STUDENT_VERSION

using System;

namespace Othello_Program
{
    class PlayBoard
    {
        private int m_Resolution;
        private GamePiece[,] m_Board;
        private int[] m_NumPieces;
        private int m_CurrentPlayer;
        private String[] m_PlayerString;

        // STUDENTS: Notice that since there are two players, and
        // since we want to refer to players using a textual name (rather than
        // a number like 0), but since C# would prefer to refer to the
        // players via numbers, we instead use a trick of defining
        // a 'variable' with the number 0, and in our code, we use 
        // the variable name instead of 0.
        // (Note that the variable is marked as being 'const', meaning 'constant'-
        // C# will never let this 'variable' change it's value)
        // So, we could use code like the following in our PlayToCell method:
        //
        //  if (CurrentPlayer() == BlackPlayer)
        //      Console.WriteLine("Black's turn to move!");
        //  else
        //      Console.WriteLine("White's turn to move!");
        //
        public const int BlackPlayer = 0;
        // STUDENTS: This is the same trick for the player who will 
        // place White game pieces (see comment above).
        public const int WhitePlayer = 1;


        public PlayBoard(int res)
        {
            m_NumPieces = new int[2];
            m_NumPieces[BlackPlayer] = 2;
            m_NumPieces[WhitePlayer] = 2;
            m_Resolution = res;
            m_Board = new GamePiece[res, res];
            m_CurrentPlayer = 0;
            for (int i = 0; i < m_Resolution; i++)
                for (int j = 0; j < m_Resolution; j++)
                    m_Board[i, j] = null;

            int mid = res / 2;
            m_Board[mid, mid - 1] = new GamePiece(BlackPlayer);
            m_Board[mid - 1, mid] = new GamePiece(BlackPlayer);
            m_Board[mid, mid] = new GamePiece(WhitePlayer);
            m_Board[mid - 1, mid - 1] = new GamePiece(WhitePlayer);

            m_PlayerString = new String[2];
            m_PlayerString[BlackPlayer] = "Black";
            m_PlayerString[WhitePlayer] = "White";
        }

        public void DrawBoard()
        {
            const int ROW_PREFIX_WIDTH = 4;
            const int COL_WIDTH = 6;

            DrawColumnHeader();

            drawLine(ROW_PREFIX_WIDTH + m_Resolution * COL_WIDTH);

            for (int iRow = m_Resolution - 1; iRow >= 0 ; iRow--)
            {
                drawEmptyRow(m_Resolution);

                Console.Write("{0,-3}|", iRow); // row # 'prefix'
                for (int iCol = 0; iCol < m_Resolution; iCol++)
                {
                    if (m_Board[iRow, iCol] == null)
                        Console.Write("     |");
                    else
                    {
                        Console.Write("  ");
                        m_Board[iRow, iCol].DrawPiece();
                        Console.Write("  |");
                    }
                }
                Console.WriteLine();

                drawEmptyRow(m_Resolution);

                drawLine(ROW_PREFIX_WIDTH + m_Resolution * COL_WIDTH);
            }
            DrawColumnHeader();

            Console.WriteLine("\n\n");
        }

        private void DrawColumnHeader()
        {
            Console.Write("   |");
            for (int iCol = 0; iCol < m_Resolution; iCol++)
            {
                Console.Write("  {0,-2} |", iCol);
            }
            Console.WriteLine();
        }

        private void drawEmptyRow(int numCols)
        {
            Console.Write("   |"); // row 'prefix'
            for (int iCol = 0; iCol < numCols; iCol++)
            {
                Console.Write("     |");
            }
            Console.WriteLine();
        }

        private void drawLine(int len)
        {
            for (int i = 0; i < len; i++)
                Console.Write("-");
            Console.WriteLine();
        }

        private bool InRange(int index)
        {
            return ((index >= 0) && (index < m_Resolution));
        }


        ///////////////////////////////////////////////////////////////////////
        // FOR STUDENT STARTER PROJECT: REMOVE CODE BELOW, STARTING HERE     //
#if INSTRUCTOR_VERSION/////////////////////////////////////////////////////////

        //
        // Find Along the column, so we look at neighbouring rows
        //

        private int FindVerticalEnd(int row, int column, int delta)
        {
            bool foundEnd = false;
            bool foundEmpty = false;
            int endIndex = row;
            while (InRange(endIndex + delta) && (!foundEnd) && (!foundEmpty))
            {
                endIndex += delta;
                foundEmpty = (null == m_Board[endIndex, column]);
                if (!foundEmpty)
                    foundEnd = (m_Board[endIndex, column].IsPlayer(CurrentPlayer()));

            }

            return foundEnd ? endIndex : row;
        }


        private int FlipAlongColumnRange(int rowStart, int rowEnd, int column)
        {
            int count = 0;
            for (int i = rowStart; i < rowEnd; i++)
            {
                m_Board[i, column].FlipToPlayer(CurrentPlayer());
                count++;
            }
            return count;
        }

        private bool FlipColumn(int row, int column)
        {
            //
            // 1st, check +ve vertical
            //
            int endIndex = FindVerticalEnd(row, column, 1);
            int count = FlipAlongColumnRange(row + 1, endIndex, column);

            //
            // now check for -ve horizontal
            int startIndex = FindVerticalEnd(row, column, -1);
            count += FlipAlongColumnRange(startIndex + 1, row, column);

            m_NumPieces[CurrentPlayer()] += count;
            m_NumPieces[NextPlayer()] -= count;

            return (count > 0);
        }


        private bool CheckColumn(int row, int column)
        {
            int endIndex = FindVerticalEnd(row, column, 1);
            bool found = (endIndex > (row + 1));

            if (!found)
            {
                int startIndex = FindVerticalEnd(row, column, -1);
                found = row > (startIndex + 1);
            }
            return found;
        }

        private int FlipAlongRowRange(int row, int colStart, int colEnd)
        {
            int count = 0;
            for (int i = colStart; i < colEnd; i++)
            {
                m_Board[row, i].FlipToPlayer(CurrentPlayer());
                count++;
            }
            return count;
        }

        /////////////////////////////////////////////////////////////////////////////////
        // Function: FindHorizontalEnd()
        //
        //    starting from position (row, column) search in 
        //    delta-direction (1 or -1) for the first piece
        //    that is also "currentPlayer".
        //
        // Return: The column index of the position (if none is found, returned
        //          column index is the same as the input column index)
        // 
        // Note: "static" makes this function local to this file.
        //
        /////////////////////////////////////////////////////////////////////////////////
        private int FindHorizontalEnd(int row, int column, int delta)
        {
            bool foundEnd = false;		// indicating have found the end piece
            bool foundEmpty = false;	// indicating have found an empty space
            int endIndex = column;		// this is the returned end index

            while (InRange(endIndex + delta) && (!foundEnd) && (!foundEmpty))
            {
                endIndex += delta;
                foundEmpty = (null == m_Board[row, endIndex]);
                if (!foundEmpty)
                    foundEnd = (m_Board[row, endIndex].IsPlayer(CurrentPlayer()));

            }

            return foundEnd ? endIndex : column;
        }

        private bool FlipRow(int row, int column)
        {
            //
            // 1st, check +ve horizontal
            //
            int endIndex = FindHorizontalEnd(row, column, 1);
            int count = FlipAlongRowRange(row, column + 1, endIndex);

            //
            // now check for -ve horizontal
            int startIndex = FindHorizontalEnd(row, column, -1);
            count += FlipAlongRowRange(row, startIndex + 1, column);

            m_NumPieces[CurrentPlayer()] += count;
            m_NumPieces[NextPlayer()] -= count;

            return (count > 0);
        }

        private bool CheckRow(int row, int column)
        {
            // 1st, check +ve direction
            //
            int endIndex = FindHorizontalEnd(row, column, 1);
            bool found = (endIndex > (column + 1));
            if (!found)
            {
                //
                // only check -ve direction is +ve do not have any result
                int startIndex = FindHorizontalEnd(row, column, -1);
                found = column > (startIndex + 1);
            }
            return found;
        }

        private void FlipAlongDiagRange(int row, int column, int count, int deltaR, int deltaC)
        {
            while (count > 0)
            {
                row += deltaR;
                column += deltaC;
                m_Board[row, column].FlipToPlayer(CurrentPlayer());
                count--;

            }
        }

        //
        // Find Along the column, so we look at neighbouring rows
        //
        private
        int CountDiagFlips(int row, int column, int deltaR, int deltaC)
        {
            bool foundEnd = false;
            bool foundEmpty = false;
            int count = -1;
            while (InRange(row + deltaR) && InRange(column + deltaC) && (!foundEnd) && (!foundEmpty))
            {
                row += deltaR;
                column += deltaC;
                foundEmpty = (null == m_Board[row, column]);
                if (!foundEmpty)
                    foundEnd = (m_Board[row, column].IsPlayer(CurrentPlayer()));
                count++;
            }

            return foundEnd ? count : 0;
        }

        private bool FlipDiagonal(int row, int column)
        {
            bool once = false;
            int count;
            int[] deltaR = { 1, -1 };
            int[] deltaC = { 1, -1 };

            for (int i = 0; i < 2; i++)
            {
                for (int j = 0; j < 2; j++)
                {
                    count = CountDiagFlips(row, column, deltaR[i], deltaC[j]);
                    if (count > 0)
                    {
                        FlipAlongDiagRange(row, column, count, deltaR[i], deltaC[j]);
                        m_NumPieces[CurrentPlayer()] += count;
                        m_NumPieces[NextPlayer()] -= count;
                        once = true;
                    }
                }
            }

            return once;
        }

        private bool CheckDiagonal(int row, int column)
        {
            bool once = false;
            int count;
            int[] deltaR = { 1, -1 };
            int[] deltaC = { 1, -1 };
            int i = 0;
            int j;

            while ((!once) && (i < 2))
            {
                j = 0;
                while ((!once) && (j < 2))
                {
                    count = CountDiagFlips(row, column, deltaR[i], deltaC[j]);
                    once = (count > 0);
                    j++;
                }
                i++;
            }

            return once;
        }

        /// <summary>
        ///    PlayToCell is given the cell/square that the current player wishes to
        ///     place one of their pieces into, as specified by (X,Y) (or rather, by (col, row))
        ///     location.  If the move is valid, this method will flip any pieces that need
        ///     to be flipped, and then return true.  If the move is NOT valid (for any 
        ///     reason), it will return false.
        /// 
        /// This method is one of the two that the students
        ///    are required to implement.  The instructor
        ///    solution uses functional decomposition/modularization extensively.
        /// 
        /// </summary>
        /// <param name="column">This is the 'X' parameter - the column of the square that
        /// the player has attempted to put a piece into.  Your code should be able to
        /// handle values that are outside of the array's bounds (less than 0, or greater than
        /// (or equal to) m_Resolution)</param>
        /// <param name="row">This is the 'Y' parameter - the row of the square that
        /// the player has attempted to put a piece into.  Your code should be able to
        /// handle values that are outside of the array's bounds (less than 0, or greater than
        /// (or equal to) m_Resolution)</param>
        /// <returns>true if the move is valid.  A valid move is described in the homework
        /// assignmetn.  This method will also flip over any pieces
        /// that need to be flipped (remember that every valid move results in pieces being
        /// flipped over).  If the current player has attemped to place a piece illegally
        /// (outside the bounds of the array, not adjacent to a previously placed piece,
        /// on a square the won't flip any of the opposing player's pieces, or for any
        /// other reason), then this method will return false.</returns>
        public bool PlayToCell(int row, int column)
        {
            bool success = false;

            if (null == m_Board[row, column])
            {
                bool hFlip = FlipRow(row, column);
                bool vFlip = FlipColumn(row, column);
                bool diagFlip = FlipDiagonal(row, column);

                success = hFlip || vFlip || diagFlip;

                if (success)
                {
                    m_Board[row, column] = new GamePiece(CurrentPlayer());
                    m_NumPieces[CurrentPlayer()]++;
                }
            }
            return success;
        }

        /// <summary>
        /// 
        /// This method will check if there are any more legal moves that the 
        ///	  "currentPlayer" can make.  Returns true if there are, returns false
        ///   if not (the game will then end afterwards)
        /// 
        /// This method is one of the two that the students
        ///    are required to implement.  The instructor
        ///    solution uses functional decomposition/modularization extensively.
        /// 
        /// </summary>
        /// <returns>Returns true if there are any legal moves for the current
        /// player to make, returns false if there aren't any legal moves
        /// for the current player to make </returns>
        public bool HasValidPositions()
        {
            bool hasOne = false;
            bool full = ((m_NumPieces[0] + m_NumPieces[1]) >= (m_Resolution * m_Resolution));
            int i = 0;
            int j;

            if (!full)
            {
                while ((!hasOne) && (i < m_Resolution))
                {
                    j = 0;
                    while ((!hasOne) && (j < m_Resolution))
                    {

                        hasOne = (null == m_Board[i, j]) &&
                               (CheckRow(i, j) ||
                                CheckColumn(i, j) ||
                                CheckDiagonal(i, j));

                        j++;
                    }
                    i++;
                }
            }
            return hasOne;
        }
#endif  ///////////////////////////////////////////////////////////////////////
        // FOR STUDENT STARTER PROJECT: REMOVE ABOVE CODE, ENDING HERE       //
        ///////////////////////////////////////////////////////////////////////

        ///////////////////////////////////////////////////////////////////////
#if STUDENT_VERSION////////////////////////////////////////////////////////////////
        // STUDENTS: YOU NEED TO WRITE YOUR CODE BELOW:

        /// <summary>
        ///    PlayToCell is given the cell/square that the current player wishes to
        ///     place one of their pieces into, as specified by (X,Y) (or rather, by (col, row))
        ///     location.  If the move is valid, this method will flip any pieces that need
        ///     to be flipped, and then return true.  If the move is NOT valid (for any 
        ///     reason), it will return false.
        /// 
        /// This method is one of the two that the students
        ///    are required to implement.  The instructor
        ///    solution uses functional decomposition/modularization extensively.
        /// 
        /// </summary>
        /// <param name="column">This is the 'X' parameter - the column of the square that
        /// the player has attempted to put a piece into.  Your code should be able to
        /// handle values that are outside of the array's bounds (less than 0, or greater than
        /// (or equal to) m_Resolution)</param>
        /// <param name="row">This is the 'Y' parameter - the row of the square that
        /// the player has attempted to put a piece into.  Your code should be able to
        /// handle values that are outside of the array's bounds (less than 0, or greater than
        /// (or equal to) m_Resolution)</param>
        /// <returns>true if the move is valid.  A valid move is described in the homework
        /// assignment.  This method will also flip over any pieces
        /// that need to be flipped (remember that every valid move results in pieces being
        /// flipped over).  If the current player has attempted to place a piece illegally
        /// (outside the bounds of the array, not adjacent to a previously placed piece,
        /// on a square the won't flip any of the opposing player's pieces, or for any
        /// other reason), then this method will return false.</returns>
        public bool PlayToCell(int column, int row)
        {
            bool success = false;


            return success;
        }

        /// <summary>
        /// 
        /// This method will check if there are any more legal moves that the 
        ///	  "currentPlayer" can make.  Returns true if there are, returns false
        ///   if not (the code that has been provided to you will will then end 
        ///   the game afterwards)
        /// 
        /// This method is one of the two that the students
        ///    are required to implement.  The instructor
        ///    solution uses functional decomposition/modularization extensively.
        /// 
        /// </summary>
        /// <returns>Returns true if there are any legal moves for the current
        /// player to make, returns false if there aren't any legal moves
        /// for the current player to make.</returns>
        public bool HasValidPositions()
        {
            bool hasOne = false;


            // STUDENTS: Here is some example code, to help
            // demonstrate how to use some of the methods that are given
            // to you, in the provided code.
            //
            // You may want to comment this stuff out, so that you still
            // have the examples around, but they won't get in the way
            // of your coding.

            if (CurrentPlayer() == BlackPlayer)
                Console.WriteLine("Black's turn to move!");
            else
                Console.WriteLine("White's turn to move!");


            //STUDENTS:  Note that if you want to tell if a cell/square is blank
            // you'll need to check if that location is null, like so:
            if (m_Board[0, 0] == null)
                Console.WriteLine("(0,0) is empty!");
            else
                Console.WriteLine("(0,0) is occupied by a GamePiece object!");

            // STUDENTS: If you want to check if the current player
            // owns the game piece at (4,4), you can try something
            // like this:
            if(m_Board[4,4] != null && m_Board[4,4].IsPlayer(CurrentPlayer()))
                Console.WriteLine(" The current player owns the piece at (4,4)");
            else
                Console.WriteLine(" The NEXT player owns the piece at (4,4)");


            return hasOne;
        }

        //
        // STUDENTS: YOU NEED TO WRITE YOUR CODE ABOVE:
#endif  ///////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////


        // STUDENTS:  This method will return an integer, representing the
        // current player.  BlackPlayer is defined as 0 (see the comments
        // around the BlackPlayer variable, at the top of this file), and
        // WhitePlayer is defined at 1, but you should use the BlackPlayer/
        // WhitePlayer variables to refer to players, if you need to refer
        // to specific players at all.  More likely, you'll use feed this
        // method's return value to the GamePiece.IsPlayer method, in order
        // to check if a particular game piece belongs to the current player
        // or not
        private int CurrentPlayer() { return m_CurrentPlayer; }

        // STUDENTS: NextPlayer returns the integer identifying the next player
        // to play.  Since there are only two players, the next player is
        // also the only other player.  Otherwise, it works just like
        // CurrentPlayer, so read the comments about that method, too :)
        private int NextPlayer() { return ((m_CurrentPlayer + 1) % 2); }




        // The following are used by the starter project, so students
        // don't need to know about them (but are welcome to examine them, 
        // if they wish!)
        public void SwitchPlayer() { m_CurrentPlayer = NextPlayer(); }

        public int WhitePlayerHas() { return m_NumPieces[WhitePlayer]; }
        public string WhitePlayerString() { return m_PlayerString[WhitePlayer]; }
        public int BlackPlayerHas() { return m_NumPieces[BlackPlayer]; }
        public string BlackPlayerString() { return m_PlayerString[BlackPlayer]; }

        public String CurrentPlayerString() { return m_PlayerString[CurrentPlayer()]; }
        public String NextPlayerString() { return m_PlayerString[NextPlayer()]; }
        public String WinnerPlayerString()
        {
            if (m_NumPieces[WhitePlayer] > m_NumPieces[BlackPlayer])
                return m_PlayerString[WhitePlayer];
            else if (m_NumPieces[WhitePlayer] < m_NumPieces[BlackPlayer])
                return m_PlayerString[BlackPlayer];
            else
                return "Neither player";
        }

        public int getResolution() { return m_Resolution; }
    }
}

