#region 1. Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

// Our own library
using XNACS1Lib;
#endregion

namespace Othello_Program
{
    /// <summary>
    ///  This is the main class. Notice this class must subclass from the XNACS1Base
    /// </summary>
    public class Othello : XNACS1Base
    {
        // Private application state
        // Location and size of the world
        private const float WORLD_MIN_X = -8.0f;          // lower left corner is (0, 0)
        private const float WORLD_MIN_Y = -1.0f;
        private const float WORLD_WIDTH = 30.0f;        // width of the world is 100.0

        private const int BOARD_RESOLUTION = 6; // 16;

        private XNACS1Circle m_Pointer;
        private Vector2 m_DeltaMove;
        private PlayBoard m_Board;

        private String m_Status;
        private String m_PlayerStatus;

        #region override base class methods
        /// <summary>
        /// Initialize the application state
        /// </summary>
        protected override void InitializeWorld()
        {
            World.SetWorldCoordinate(new Vector2(WORLD_MIN_X, WORLD_MIN_Y), WORLD_WIDTH);
            World.SetBackgroundColor(Color.LightSkyBlue);

            m_DeltaMove = new Vector2();
            m_DeltaMove.X = 0.0f;
            m_DeltaMove.Y = 0.0f;

            m_Board = new PlayBoard(BOARD_RESOLUTION);

            // the small red dot as current position
            Vector2 pos = new Vector2();
            pos.X = 0.5f * BOARD_RESOLUTION + 0.5f;
            pos.Y = 0.5f * BOARD_RESOLUTION + 0.5f;
            m_Pointer = new XNACS1Circle(pos, 0.1f);
            m_Pointer.CenterColor = Color.Red;
            m_Pointer.OutsideColor = Color.LightSalmon;

            m_PlayerStatus = m_Board.CurrentPlayerString() + " piece turn";
        }

        /// <summary>
        /// Update the application state. 
        /// </summary>
        protected override void UpdateWorld()
        {
            Vector2 delta = XNACS1Base.GamePad.ThumbSticks.Left;
            m_DeltaMove += (0.4f * delta);
            int dx = (int)m_DeltaMove.X;
            int dy = (int)m_DeltaMove.Y;

            if ((dx != 0) || (dy != 0))
            {
                m_DeltaMove.X = 0.0f;
                m_DeltaMove.Y = 0.0f;
                XNACS1Base.PlayACue( "Move");
            }

            m_Pointer.CenterX += dx;
            m_Pointer.CenterY += dy;


            if (XNACS1Base.GamePad.ButtonAClicked())
            {
                int row = (int)m_Pointer.CenterX;
                int col = (int)m_Pointer.CenterY;
                if ((row >= 0) && (row < BOARD_RESOLUTION) && (col >= 0) && (col < BOARD_RESOLUTION))
                {
                    if (m_Board.PlayToCell(col, row))
                    {
                        int w = m_Board.WhitePlayerHas();
                        int b = m_Board.BlackPlayerHas();

                        m_Status = "Player Pieces: White=" + w + "  Black=" + b;
                        XNACS1Base.PlayACue( "Click");

                        m_Board.SwitchPlayer();
                        m_PlayerStatus = m_Board.CurrentPlayerString() + " piece turn";

                        if (!m_Board.HasValidPositions())
                        {
                            m_Status += "   ****Player: " + m_Board.WinnerPlayerString() + " has won!";
                        }

                    }
                    else
                    {
                        m_Status = "Position: (" + row + ", " + col + ") is not a valid move. Try again!";
                        XNACS1Base.PlayACue( "Beep");
                    }
                }
                else
                {
                    m_Status = "Position: (" + row + ", " + col + ") is outside the game board. Try again!";
                }
            }

            // Allows the default game to exit on Xbox 360 and Windows
            if (XNACS1Base.GamePad.Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // must make sure the red dot is always on top ...
            m_Pointer.TopOfAutoDrawSet();

            EchoToTopStatus(m_PlayerStatus + @"    Current Position:" + m_Pointer.Center);
            EchoToBottomStatus(m_Status);
        }
        #endregion
    }
}
