#define STUDENT_VERSION

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

// Our own library
using XNACS1Lib;

namespace Othello_Program
{
    class PlayBoard
    {
        // alternates between 7 pre-defined colors
        private const float SIZE = 0.95f;
        private int m_Resolution;
        private GamePiece[,] m_Board;
        private int[] m_NumPieces;
        private int m_CurrentPlayer;
        private String[] m_PlayerString;

        // STUDENTS: Notice that since there are two players, and
        // since we want to refer to players using a textual name (rather than
        // a number like 0), but since C# would prefer to refer to the
        // players via numbers, we instead use a trick of defining
        // a 'variable' with the number 0, and in our code, we use 
        // the variable name instead of 0.
        // (Note that the variable is marked as being 'const', meaning 'constant'-
        // C# will never let this 'variable' change it's value)
        // So, we could use code like the following in our PlayToCell method:
        //
        //  if (CurrentPlayer() == BlackPlayer)
        //      XNACS1Base.EchoToBottomStatus("Black's turn to move!");
        //  else
        //      XNACS1Base.EchoToBottomStatus("White's turn to move!");
        //
        public const int BlackPlayer = 0;
        // STUDENTS: This is the same trick for the player who will 
        // place White game pieces (see comment above).
        public const int WhitePlayer = 1;


        public PlayBoard(int res)
        {
            m_NumPieces = new int[2];
            m_NumPieces[BlackPlayer] = 2;
            m_NumPieces[WhitePlayer] = 2;
            m_Resolution = res;
            m_Board = new GamePiece[res, res];
            m_CurrentPlayer = 0;

            Vector2 pos = new Vector2();
            for (int i = 0; i < m_Resolution; i++)
            {
                pos.X = i + 0.5f;
                for (int j = 0; j < m_Resolution; j++)
                {
                    pos.Y = j + 0.5f;
                    m_Board[i, j] = null;
                    XNACS1Rectangle cell = new XNACS1Rectangle(pos, SIZE, SIZE);
                    cell.OutsideColor = cell.CenterColor = Color.LightSeaGreen;
                }
            }

            int mid = res / 2;
            m_Board[mid, mid - 1] = new GamePiece(mid, mid - 1, WhitePlayer);
            m_Board[mid - 1, mid] = new GamePiece(mid - 1, mid, WhitePlayer);
            m_Board[mid, mid] = new GamePiece(mid, mid, BlackPlayer);
            m_Board[mid - 1, mid - 1] = new GamePiece(mid - 1, mid - 1, BlackPlayer);

            m_PlayerString = new String[2];
            m_PlayerString[BlackPlayer] = "Black";
            m_PlayerString[WhitePlayer] = "White";
        }

        private bool InRange(int index)
        {
            return ((index >= 0) && (index < m_Resolution));
        }


        ///////////////////////////////////////////////////////////////////////
#if STUDENT_VERSION////////////////////////////////////////////////////////////////
        // STUDENTS: YOU NEED TO WRITE YOUR CODE BELOW:

        /// <summary>
        ///    PlayToCell is given the cell/square that the current player wishes to
        ///     place one of their pieces into, as specified by (X,Y) (or rather, by (col, row))
        ///     location.  If the move is valid, this method will flip any pieces that need
        ///     to be flipped, and then return true.  If the move is NOT valid (for any 
        ///     reason), it will return false.
        /// 
        /// This method is one of the two that the students
        ///    are required to implement.  The instructor
        ///    solution uses functional decomposition/modularization extensively.
        /// 
        /// </summary>
        /// <param name="column">This is the 'X' parameter - the column of the square that
        /// the player has attempted to put a piece into.  Your code should be able to
        /// handle values that are outside of the array's bounds (less than 0, or greater than
        /// (or equal to) m_Resolution)</param>
        /// <param name="row">This is the 'Y' parameter - the row of the square that
        /// the player has attempted to put a piece into.  Your code should be able to
        /// handle values that are outside of the array's bounds (less than 0, or greater than
        /// (or equal to) m_Resolution)</param>
        /// <returns>true if the move is valid.  A valid move is described in the homework
        /// assignment.  This method will also flip over any pieces
        /// that need to be flipped (remember that every valid move results in pieces being
        /// flipped over).  If the current player has attempted to place a piece illegally
        /// (outside the bounds of the array, not adjacent to a previously placed piece,
        /// on a square the won't flip any of the opposing player's pieces, or for any
        /// other reason), then this method will return false.</returns>
        public bool PlayToCell(int column, int row)
        {
            bool success = false;

           
            return success;
        }

        /// <summary>
        /// 
        /// This method will check if there are any more legal moves that the 
        ///	  "currentPlayer" can make.  Returns true if there are, returns false
        ///   if not (the code that has been provided to you will will then end 
        ///   the game afterwards)
        /// 
        /// This method is one of the two that the students
        ///    are required to implement.  The instructor
        ///    solution uses functional decomposition/modularization extensively.
        /// 
        /// </summary>
        /// <returns>Returns true if there are any legal moves for the current
        /// player to make, returns false if there aren't any legal moves
        /// for the current player to make.</returns>
        public bool HasValidPositions()
        {
            bool hasOne = false;


            // STUDENTS: Here is some example code, to help
            // demonstrate how to use some of the methods that are given
            // to you, in the provided code.
            //
            // You may want to comment this stuff out, so that you still
            // have the examples around, but they won't get in the way
            // of your coding.

            if (CurrentPlayer() == BlackPlayer)
                XNACS1Base.EchoToBottomStatus("Black's turn to move!");
            else
                XNACS1Base.EchoToBottomStatus("White's turn to move!");

            string topMsg = "";

            //STUDENTS:  Note that if you want to tell if a cell/square is blank
            // you'll need to check if that location is null, like so:
            if (m_Board[0, 0] == null)
                topMsg += "(0,0) is empty!";
            else
                topMsg += "(0,0) is occupied by a GamePiece object!";

            // STUDENTS: If you want to check if the current player
            // owns the game piece at (4,4), you can try something
            // like this:
            if(m_Board[4,4] != null && m_Board[4,4].IsPlayer(CurrentPlayer()))
                topMsg += " The current player owns the piece at (4,4)";
            else
                topMsg += " The NEXT player owns the piece at (4,4)";

            XNACS1Base.EchoToTopStatus(topMsg);


            return hasOne;
        }

        //
        // STUDENTS: YOU NEED TO WRITE YOUR CODE ABOVE:
#endif  ///////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////////////////////////


        // STUDENTS:  This method will return an integer, representing the
        // current player.  BlackPlayer is defined as 0 (see the comments
        // around the BlackPlayer variable, at the top of this file), and
        // WhitePlayer is defined at 1, but you should use the BlackPlayer/
        // WhitePlayer variables to refer to players, if you need to refer
        // to specific players at all.  More likely, you'll use feed this
        // method's return value to the GamePiece.IsPlayer method, in order
        // to check if a particular game piece belongs to the current player
        // or not
        private int CurrentPlayer() { return m_CurrentPlayer; }

        // STUDENTS: NextPlayer returns the integer identifying the next player
        // to play.  Since there are only two players, the next player is
        // also the only other player.  Otherwise, it works just like
        // CurrentPlayer, so read the comments about that method, too :)
        private int NextPlayer() { return ((m_CurrentPlayer + 1) % 2); }




        // The following are used by the starter project, so students
        // don't need to know about them (but are welcome to examine them, 
        // if they wish!)
        public void SwitchPlayer() { m_CurrentPlayer = NextPlayer(); }

        public int WhitePlayerHas() { return m_NumPieces[WhitePlayer]; }
        public int BlackPlayerHas() { return m_NumPieces[BlackPlayer]; }

        public String CurrentPlayerString() { return m_PlayerString[CurrentPlayer()]; }
        public String WinnerPlayerString()
        {
            if (m_NumPieces[WhitePlayer] > m_NumPieces[BlackPlayer])
                return m_PlayerString[WhitePlayer];
            else
                return m_PlayerString[BlackPlayer];
        }

    }
}