using System;
using System.Text;

// If you haven't done so already, you should read the README.txt
// file, first


// In retrospect, making a class that will compartmentalize
// the logic behind a Console User Interface (CUI) menu, when the
// program has only 1 menu, was kinda silly.  D'oh!
//
// The upside is that it does serve as a good example of how one 
// might encapsulate this sort of logic....
namespace StarterProject
{
	public delegate bool FnxToCall(Player p, BurstABubble game);

	public struct menuItem
    {
        public string m_text;
        public FnxToCall m_action;
        public bool m_active;

        public menuItem(string text, FnxToCall action)
        {
            m_text = text;
            m_action = action;
            m_active = true;
        }
    }

	public class MenuCUI
	{
		private menuItem[] m_menu;

		/// <summary>
		/// 
		/// </summary>
		/// <param name="items">A reference to an array of menuItem objects
		/// The array should not have any 'blank' items in it.
		/// This object will take possession (ownership) of the block of memory
		/// - in other words, once you've passed the argument to this constructor,
		/// you should no longer add to, modify, remove, examine, or do anything else
		/// to the array.
		/// </param>
		public MenuCUI(menuItem[] items)
		{
			m_menu = items;
		}

		public int GetUserChoice()
		{
			return GetValidUserInput(getMenuAsString(), 0, m_menu.Length); ;
		}

		public void PrintMenu()
		{
			Console.WriteLine(getMenuAsString());
		}

		private string getMenuAsString()
		{
			string s = "Please select from the following:\n";
			for (int i = 0; i < m_menu.Length; i++)
			{
				if (m_menu[i].m_active)
					s = String.Concat(s, i,":\t", m_menu[i].m_text, "\n");
			}
			return String.Concat(s, "Your choice?\n");
		}

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		public bool GetAndExecuteMenuChoice(Player p, BurstABubble game)
		{
			return m_menu[ GetUserChoice() ].m_action(p, game);
		}

		static public int GetValidUserInput(string s, int min, int max)
		{
			if (min == Int32.MinValue)
				throw new Exception("min needs to be " + (Int32.MinValue + 1) + " or higher!");

			int iChoice = min - 1;
			while (iChoice < min || iChoice >= max)
			{
				try
				{
					Console.WriteLine(s);
					iChoice = Int32.Parse(Console.ReadLine());
					if (iChoice < min)
						Console.WriteLine("You have to choose a number higher than {0}", min-1);
					else if (iChoice >= max)
						Console.WriteLine("You have to choose a number lower than {0}", max);
				}
				catch (Exception) // variable name omitted to avoid compiler warning
				{
					Console.WriteLine("You need to type in a number!");
					iChoice = min - 1;
				}
			}
			return iChoice;
		}

	}
}
