using System;
using System.Collections.Generic;
using System.Text;

namespace Helpdesk
{
    class Helpdesk_SingleQueue
    {
        /// <summary>
        /// Copied from The ListOfTickets, which is bad :)
        /// Added 'Priority' field to this version
        /// </summary>
        private class ListNode
        {
            public ListNode m_next;
            public Ticket m_ticket;

            public ListNode(string desc, Priority prio)
            {
                m_ticket = new Ticket(desc, prio);
            }
        }

        // m_first:
        // null if list is empty.  
        // If there are high priority items, 
        //      this will point to the front (oldest) high priority item
        // If there are only Low priority items, 
        //      this will point to the front (oldest) Low priority item
        private ListNode m_first;

        // m_lastHi:
        // null if there are no high priority items
        // otherwise, points to the last item in the 'high priority' queue
        private ListNode m_lastHi;

        // m_lastLo:
        // null if there are no Low priority items
        // otherwise, points to the last item in the 'Low priority' queue
        private ListNode m_lastLo;

        public bool isEmpty()
        {
            return m_first == null;
        }

        public void AddTicket(string desc, Priority prio)
        {
            if(prio != Priority.High && prio != Priority.Low)
                    throw new ApplicationException("Somehow given a priority other than Low or High!");

            ListNode newNode = new ListNode(desc, prio);

            // If there are no tickets whatsoever
            if (null == m_first)
            {
                // the new ticket will be first
                m_first = newNode;

                // Also set up the hi/lo priority 'end of list' markers
                if (Priority.High == prio)
                {
                    m_lastHi = newNode;
                }
                else if (Priority.Low == prio)
                {
                    m_lastLo = newNode;
                }
            }
            else
            {
                // There is at least one item in the list

                if (Priority.High == prio)
                {
                    // If there aren't any previous Hi prios...
                    if (null == m_lastHi)
                    {
                        // ... then we need to remember that this is the first
                        m_lastHi = newNode;

                        // Also add to the front, in front of the 1+ Low priority tickets
                        newNode.m_next = m_first;
                        m_first = newNode;
                    }
                    else
                    {
                        // There's at least one other high priority item,
                        // and zero (or more) Low priority items
                        newNode.m_next = m_lastHi.m_next;
                        m_lastHi.m_next = newNode;
                        m_lastHi = newNode;
                    }
                }
                else if (Priority.Low == prio)
                {
                    if (null == m_lastLo)
                    {
                        // If we're here, there must be at least one (high-priority) ticket
                        // and no Low priority tickets
                        System.Diagnostics.Debug.Assert(m_lastHi != null);

                        // new node is now the end of the 'Low priority' part of the queue
                        m_lastLo = newNode;

                        // high priority part of queue will point to the Low priority ticket(s)...
                        m_lastHi.m_next = newNode;
                    }
                    else
                    {
                        // we have at least one Low priority ticket
                        // so just add to end
                        m_lastLo.m_next = newNode;
                        m_lastLo = newNode;
                    }
                }
            }
        }

        public Ticket RemoveNextTicket()
        {
            if (null == m_first)
            {
                // no tickets to remove!
                return null;
            }
            else if (m_first == m_lastHi) // about to remove last Hi prio item
            {
                m_lastHi = null;
            }
            else if (m_first == m_lastLo) // about to remove last Low prio item
            {
                m_lastLo = null;
            }
            // else we're in the beginning/middle of either the High or Low priority items
            // and nothing special needs to be done

            Ticket retVal = m_first.m_ticket;
            m_first = m_first.m_next;
            return retVal;
        }

        public void PrintAll()
        {
            Console.WriteLine("Current Helpdesk tickets:");
            
            ListNode cur = m_first;

            if(m_lastHi == null)
            {
                Console.WriteLine("\tNo high priority tasks");
            }
            else
            {
                Console.WriteLine("\tHigh Priority Tasks:");
                while (cur != null && cur != m_lastHi.m_next)
                {
                    cur.m_ticket.Print();
                    cur = cur.m_next;
                }
            }

            if (m_lastLo == null)
            {
                Console.WriteLine("\tNo low priority tasks");
            }
            else
            {
                Console.WriteLine("\tLow Priority Tasks:");
                while (cur != null)
                {
                    cur.m_ticket.Print();
                    cur = cur.m_next;
                }
            }
        }
    }
}
