using System;


namespace Helpdesk
{

    enum Priority
    {
        Low,
        High
    }

    class ListOfTickets
    {
        // null if the queue is empty, otherwise points to the front
        // item in the queue (to the next item that will be removed)
        ListNode m_first;

        // If m_first is null, m_last is undefined, and may very well be *non-null*
        // _even_if_the_list_is_empty_!
        //
        // Otherwise it points to the back/rear of the queue, and is
        // where new items will be added
        ListNode m_last;


        /// <summary>
        /// C#-ism: This is a nested class.  It's similar to a 
        /// Java 'static inner class'.  Private means that it's not
        /// accessible outside of the enclosing LinkedList class, but
        /// unlike a Java non-static inner class, each instance of the 'Node
        /// isn't tied to an instance of the LinkedList class.
        /// 
        /// We could also make this into a top-level class (in which case,
        /// we should mark all the data fields as being private, and expose
        /// them via properties/accessor&mutator methods, instead.
        /// </summary>
        private class ListNode
        {
            public ListNode m_next;
            public Ticket m_ticket;

            public ListNode(Ticket t)
            {
                m_ticket = t;
            }
        }

        public ListOfTickets()
        {
            // m_first is initialized to null, so no need to do anything...
        }

        /// <summary>
        /// Creates a new 'node, and pre-pends it to the front of the list
        /// </summary>
        /// <param name="desc"></param>
        public void AddTicket(Ticket t)
        {
            // Create the node
            ListNode newNode = new ListNode(t);

            // Add it to the end of the list
            
            if (null == m_first)
            {
                // If m_last is null, then the list is empty
                m_first = m_last = newNode;
                return;
            }
            else
            {
                // List is not empty, so tack it onto the end
                m_last.m_next = newNode;
                m_last = newNode;
            }
        }

        public Ticket RemoveNextTicket()
        {
            // If there's something in the list
            if (null != m_last)
            {
                Ticket retVal = m_first.m_ticket;

                // move m_first up
                m_first = m_first.m_next;

                // If m_first is now null, we'll correct no-op on any future
                // RemoveNextTicket calls, and in AddTicket, we'll re-set
                // everything

                return retVal;
            }
            else
                return null;
        }

        public void PrintAll()
        {
            ListNode cur = m_first;
            while (null != cur)
            {
                cur.m_ticket.Print();
                cur = cur.m_next;
            }
        }

        public void PrintFirst()
        {
            if (null != m_first)
                m_first.m_ticket.Print();
        }

        public bool isEmpty()
        {
            return m_first == null;
        }
    }
}
