using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

using XNACS1Lib;
using ListOfToysBase;

namespace CatchToyGame_Program
{
    // NOTE: IN ORDER FOR THIS PROGRAM TO WORK CORRECTLY, YOU NEED
    // TO HAVE YOUR ListOfToys CLASS INHERIT FROM CListOfToysBase
    //  (The IterateAndCompute method is defined on that base class)
    class ListOfToys : CListOfToysBase
    {
        // NOTE: IN ORDER FOR THIS PROGRAM TO WORK CORRECTLY, YOU NEED
        // TO HAVE YOUR ListNode CLASS IMPLEMENT IListOfToysNode
        private class ListNode : IListOfToysNode
        {
            public ListNode m_Next;
            public IListOfToysNode Next
            {
                get
                {
                    return m_Next;
                }
            }

            public ToyObject m_Object;
            public float GetNodeSize 
            { 
                 get 
                 { 
                      return m_Object.GetToySize; 
                 } 
            }

            public void SetNodePosition(float x, float y)
            { 
                  m_Object.SetToyPosition(x, y); 
            }

            public ListNode(ToyDropSpeed p)
            {
                m_Object = new ToyObject(p);
                m_Next = null;
            }
        }

        // NOTE: IN ORDER FOR THIS PROGRAM TO WORK CORRECTLY, YOU NEED
        // TO HAVE YOUR ListOfToys CLASS IMPLEMENT GetNextNode
        //  (The IterateAndCompute method uses the IListOfToysNode to
        //  iterate through all the items in the list, by using the
        //  GetNextNode method to go through the list, and the GetToy property
        //  to get the ToyObject that this node keeps track of.)
        // The list should store the oldest, high-speed item first,
        // then the second-oldest, high-speed item, etc, etc
        // Then the oldest low-speed item, second oldest high-speed item, etc, etc
        //
        // This method is here in case you don't want to have a single, long list
        //  of all the Toys, for whatever reason.
        protected override IListOfToysNode GetNextNode(IListOfToysNode current)
        {
            if (current != null)
                return current.Next;
            else
                return null;
        }

        private int m_Count;

        private ListNode m_First;
        private ListNode m_LastHi;
        private ListNode m_LastLo;

        public ListOfToys()
        {
            m_Count = 0;

            m_First = null;
            m_LastHi = null;
            m_LastLo = null;
        }

        public void AddToy(ToyDropSpeed s)
        {
            ListNode newNode = new ListNode(s);
            m_Count++;

            // if empty queue
            if (null == m_First)
            {
                // the new ticket will be first
                m_First = newNode;

                // Also set up the hi/lo speed 'end of list' markers
                if (ToyDropSpeed.High == s)
                {
                    m_LastHi = newNode;
                }
                else if (ToyDropSpeed.Low == s)
                {
                    m_LastLo = newNode;
                }
            }
            else
            {
                // There is at least one item in the list

                if (ToyDropSpeed.High == s)
                {
                    // If there aren't any previous Hi prios...
                    if (null == m_LastHi)
                    {
                        // ... then we need to remember that this is the first
                        m_LastHi = newNode;

                        // Also add to the front, in front of the 1+ low speed tickets
                        newNode.m_Next = m_First;
                        m_First = newNode;
                    }
                    else
                    {
                        // There's at least one other high speed item,
                        // and zero (or more) low speed items
                        newNode.m_Next = m_LastHi.m_Next;
                        m_LastHi.m_Next = newNode;
                        m_LastHi = newNode;
                    }
                }
                else if (ToyDropSpeed.Low == s)
                {
                    if (null == m_LastLo)
                    {
                        // If we're here, there must be at least one (high-speed) ticket
                        // and no low speed tickets
                        System.Diagnostics.Debug.Assert(m_LastHi != null);

                        // new node is now the end of the 'low speed' part of the queue
                        m_LastLo = newNode;

                        // high speed part of queue will point to the low speed ticket(s)...
                        m_LastHi.m_Next = newNode;
                    }
                    else
                    {
                        // we have at least one low speed ticket
                        // so just add to end
                        m_LastLo.m_Next = newNode;
                        m_LastLo = newNode;
                    }
                }
            }
            base.IterateAndCompute(XNACS1Base.World.WorldMax.Y, m_Count, m_First);
        }

        public ToyObject RemoveNextToy()
        {
            if (null == m_First)
            {
                // no more balls to remove!
                return null;
            }
            else if (m_First == m_LastHi) // about to remove last Hi prio item
            {
                m_LastHi = null;
            }
            else if (m_First == m_LastLo) // about to remove last low prio item
            {
                m_LastLo = null;
            }
            // else we're in the beginning/middle of either the High or low speed items
            // and nothing special needs to be done

            ToyObject r = m_First.m_Object;
            m_First = m_First.m_Next;
            m_Count--;

            return r;
        }

        public bool isEmpty()
        {
            return m_First == null;
        }

        public void CurrentToyCount()
        {
            int hiCount = 0;
            int loCount = 0;
            ListNode cur = m_First;

            if (null != m_LastHi)
            {
                while (cur != null && cur != m_LastHi.m_Next)
                {
                    cur = cur.m_Next;
                    hiCount++;
                }
            }

            while (cur != null)
            {
                cur = cur.m_Next;
                loCount++;
            }
            XNACS1Base.EchoToBottomStatus(@"Animals Left: " + hiCount + @"  Tools Left: " + loCount);
        }
    }
}
