
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

// Our own library
using XNACS1Lib;

#endregion


// 
//
// ALL Media files must be PART of the solution (to be deployed over to the XBOX)!!


namespace CatchToyGame_Program
{
    public enum ToyDropSpeed
    {
        Low,
        High
    };

    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class CatchToyGame : XNACS1Base
    {
        private const float WORLD_MIN_X = 0.0f;
        private const float WORLD_MIN_Y = 0.0f;
        public const float WORLD_WIDTH = 128.0f;

        private CatchBox m_CatchBox;
        private ToyObject m_TheDroppingToy;
        private ListOfToys m_Queue;

        private int m_Score;
        private int m_Missed;

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void InitializeWorld()
        {
            World.SetWorldCoordinate(new Vector2(WORLD_MIN_X, WORLD_MIN_Y), WORLD_WIDTH);

            // game state
            m_CatchBox = new CatchBox();
            m_TheDroppingToy = null;
            m_Queue = new ListOfToys();

            m_Score = 0;
            m_Missed = 0;
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        protected override void UpdateWorld()
        {
            if (XNACS1Base.GamePad.ButtonAClicked())
            {
                m_Queue.AddToy(ToyDropSpeed.Low);
                XNACS1Base.PlayACue( "bnew");
            }

            if (XNACS1Base.GamePad.ButtonBClicked())
            {
                m_Queue.AddToy(ToyDropSpeed.High);
                XNACS1Base.PlayACue( "anew");
            }


            // Allows the default game to exit on Xbox 360 and Windows
            if (XNACS1Base.GamePad.Buttons.Back == ButtonState.Pressed)
                this.Exit();

            Vector2 delta = XNACS1Base.GamePad.ThumbSticks.Left;
            m_CatchBox.MoveCatchBox(delta.X);

            if (null != m_TheDroppingToy)
            {
                if (m_CatchBox.CaughtToy(m_TheDroppingToy))
                {
                    m_TheDroppingToy.RemoveFromAutoDrawSet();

                    m_Score += m_TheDroppingToy.ToyScore;
                    m_TheDroppingToy = m_Queue.RemoveNextToy();
                    XNACS1Base.PlayACue( "netted");
                }
                else
                {
                    m_TheDroppingToy.Update();
                    if (m_TheDroppingToy.Expired())
                    {
                        m_TheDroppingToy.RemoveFromAutoDrawSet();

                        m_TheDroppingToy = m_Queue.RemoveNextToy();
                        m_Missed++;
                        XNACS1Base.PlayACue( "bdrop");
                    }
                }

            }
            else
            {
                m_TheDroppingToy = m_Queue.RemoveNextToy();
                if (null != m_TheDroppingToy)
                    XNACS1Base.PlayACue( "adrop");
            }

            m_Queue.CurrentToyCount();

            XNACS1Base.EchoToTopStatus("Total Score: " + m_Score + "    Toys Missed: " + m_Missed);
        }
    }
}
