using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

// Our own library
using XNACS1Lib;


namespace CatchToyGame_Program
{
    public class ToyObject : XNACS1Rectangle
    {
        private const float RADIUS = CatchBox.BOX_HEIGHT / 2.5f;
        private const float SPEED = CatchToyGame.WORLD_WIDTH / 120.0f; // travels the width in 3 seconds
        private const float ACCEL = 0.00001f;

        private const int LOW_P_SCORE = 1;
        private const int HIGH_P_SCORE = 5 * LOW_P_SCORE;

        private bool m_Expired;
        private int m_WorthScore;

        private const int m_TextureWrapAround = 15;
        private static int m_LoTexCount = 0;
        private static int m_HiTexCount = 0;

        public ToyObject(ToyDropSpeed p)
            :
            base(new Vector2(XNACS1Base.World.WorldMin.X / 2.0f, (CatchBox.BOX_HEIGHT * 0.5f) + RADIUS),
                2.0f * RADIUS, 2.0f * RADIUS)
        {
            float sFactor;
            Vector2 vF;
            float minY = 0.2f;
            if (ToyDropSpeed.High == p)
            {
                sFactor = 2.0f;
                vF.X = 0.4f;
                vF.Y = 1.0f;
                Texture = "a" + ToyObject.m_HiTexCount;
                ToyObject.m_HiTexCount = (ToyObject.m_HiTexCount + 1) % m_TextureWrapAround;
                m_WorthScore = HIGH_P_SCORE;
            }
            else
            {
                sFactor = 1.0f;
                vF.X = 0.2f;
                vF.Y = 1.0f;
                Texture = "b" + ToyObject.m_LoTexCount;
                ToyObject.m_LoTexCount = (ToyObject.m_LoTexCount + 1) % m_TextureWrapAround;
                m_WorthScore = LOW_P_SCORE;
            }

            VelocityX = CatchToyGame.RandomFloat(vF.X);
            VelocityY = -CatchToyGame.RandomFloat(vF.Y) - minY;
            Velocity.Normalize();
            Speed = SPEED * sFactor;

            m_Expired = false;
        }

        public void Update()
        {
            Center = Center + Velocity ;
            Speed += ACCEL;

            XNACS1Lib.BoundCollideStatus boundStatus = XNACS1Base.World.ClampAtWorldBound(this);
            switch (boundStatus)
            {
                case BoundCollideStatus.CollideBottom:
                    m_Expired = true;
                    break;

                case BoundCollideStatus.CollideTop:
                    VelocityY = -VelocityY;
                    break;

                case BoundCollideStatus.CollideLeft:
                case BoundCollideStatus.CollideRight:
                    VelocityX = -VelocityX;
                    break;
            }
        }

        public bool Expired()
        {
            return m_Expired;
        }

        public int ToyScore
        {
            get { return m_WorthScore; }
        }

        public void Caught(bool touched)
        {
            m_Expired = touched;
        }

        public void SetToyPosition(float x, float y)
        {
            CenterX = x;
            CenterY = y;
        }
        public float GetToySize { get { return RADIUS; } }
    }
}