
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

// Our own library
using XNACS1Lib;

#endregion


// 
//
// ALL Media files must be PART of the solution (to be deployed over to the XBOX)!!


namespace CatchToyGame_Program
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class CatchToyGame : XNACS1Base
    {     
        private const float WORLD_MIN_X = 0.0f;     // Lower-left corner of the
        private const float WORLD_MIN_Y = 0.0f;     //   application window is the origin
        public  const float WORLD_WIDTH = 128.0f;   // Width of window is 128 units

        private CatchBox m_CatchBox;                // Catch box at the bottom
        private ToyObject m_TheDroppingToy;         // ToyObject that is dropping
        private ListOfToys m_ToyQueue;              // All the toys

        private int m_Score;                        // Num toys caught
        private int m_Missed;                       // Num toys dropped


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void InitializeWorld()
        {
            World.SetWorldCoordinate(new Vector2(WORLD_MIN_X, WORLD_MIN_Y), WORLD_WIDTH);

            m_CatchBox = new CatchBox();        // allocate memory and 
            m_ToyQueue = new ListOfToys();
            m_TheDroppingToy = null;

            m_Score = 0;                        // initialize scores
            m_Missed = 0;

            // start with 15 toys
            for (int i = 0; i < 15; i++)
                m_ToyQueue.AddToy();
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        protected override void UpdateWorld()
        {
            // Allows the default game to exit on Xbox 360 and Windows
            if (XNACS1Base.GamePad.ButtonBackClicked())
                this.Exit();

            // Label A: add a toy if Button A is clicked
            if (XNACS1Base.GamePad.ButtonAClicked())
            {
                m_ToyQueue.AddToy();
                XNACS1Base.PlayACue( "anew");
            }

            // Label B: move catch box by left thumb stick
            Vector2 delta = XNACS1Base.GamePad.ThumbSticks.Left;
            m_CatchBox.MoveCatchBox(delta.X);

            // Label C: work with the dropping toy
            if (null != m_TheDroppingToy)
            {
                // Label C1: Current dropping top caught
                if (m_CatchBox.CaughtToy(m_TheDroppingToy))
                {
                    m_TheDroppingToy.RemoveFromAutoDrawSet();

                    m_Score++;
                    m_TheDroppingToy = m_ToyQueue.GetNextToy();
                    XNACS1Base.PlayACue( "netted");
                }
                else
                {
                    // Label C-2: topy continues to drop (off the window)
                    m_TheDroppingToy.Update();        // drop downwards
                    if (m_TheDroppingToy.Expired())   // if expired (falls outside)
                    {
                        m_TheDroppingToy.RemoveFromAutoDrawSet();

                        m_TheDroppingToy = m_ToyQueue.GetNextToy();
                        m_Missed++;
                        XNACS1Base.PlayACue( "adrop");
                    }
                }
            }
            else
            {
                // if does not have a current dropping toy, get the next one
                // from the queue
                m_TheDroppingToy = m_ToyQueue.GetNextToy();
                if (null != m_TheDroppingToy)
                    XNACS1Base.PlayACue( "anew");
            }

            m_ToyQueue.CurrentToyCount();

            XNACS1Base.EchoToTopStatus("Total Score: " + m_Score + "    toys Missed: " + m_Missed);
        }
    }
}
