using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

// Our own library
using XNACS1Lib;


namespace CatchToyGame_Program
{
    class ToyObject : XNACS1Rectangle
    {
        // Label A: Constants of toy appearance/behavior
        private const float RADIUS = CatchBox.BOX_HEIGHT/2.5f;
        private const float SPEED = CatchToyGame.WORLD_WIDTH / 120.0f; // travels the width in 3 seconds
        private const float ACCEL = 0.00001f;
        private const float INIT_X = 0.0f; // toy initial x position
        private const float INIT_Y = (CatchBox.BOX_HEIGHT * 0.5f) + RADIUS; // toy init y position
        private const float WIDTH = 2.0f * RADIUS;
        private const float HEIGHT = 2.0f * RADIUS;

        // Label B: Constants for texture file selection
        private const int m_TextureWrapAround = 15;
        private static int m_TexCount = 0;

        // Label C: Instance variables defining toy behavior
        private bool m_Expired;             // ToyObject has expired (should be remove from set)

        public ToyObject()
            : base(new Vector2(INIT_X, INIT_Y), WIDTH, HEIGHT)
        {
            // Label D: Select texture file name
            Texture = "a" + ToyObject.m_TexCount;
            ToyObject.m_TexCount = (ToyObject.m_TexCount + 1) % m_TextureWrapAround;
            
            // Label E: initialial velocity and position
            VelocityX = 2.0f;
            VelocityY = -1.0f;
            
            m_Expired = false;      // not caught and not fell off the screen yet
        }

        public void Update()
        {
            Center = Center + Velocity;           // change position with (downward) velocity
            VelocityY -= ACCEL;                  // increase downward velocity

            // determine bouncing and expiration ... 
            XNACS1Lib.BoundCollideStatus boundStatus = XNACS1Base.World.ClampAtWorldBound(this);
            switch (boundStatus)
            {
                case BoundCollideStatus.CollideBottom:
                    m_Expired = true;
                    break;

                case BoundCollideStatus.CollideTop:
                    VelocityY = -VelocityY;
                    break;

                case BoundCollideStatus.CollideLeft:
                case BoundCollideStatus.CollideRight:
                    VelocityX = -VelocityX;
                    break;
            }
        }

        public bool Expired()
        {
            return m_Expired;
        }

        public void ComputePosition(int total, int n)
        {
            CenterY = XNACS1Base.World.WorldMax.Y - (2.2f * RADIUS);
            CenterX = ((total - n) * 2 * RADIUS) - RADIUS;
        }

        public void Caught(bool touched)
        {
            m_Expired = touched;
        }
    }
}