using System;
using System.Collections.Generic;

namespace BSTSolution
{
    public class CBinarySearchTree
    {
        public enum ErrorCode
        {
            Success = 0,
            AleadyInTree = 1
        }

        private class CTreeNode
        {
            private CTreeNode m_LeftChild, m_RightChild;
            private char m_Data;

            public CTreeNode(char data)
            {
                m_Data = data;
                m_LeftChild = null;
                m_RightChild = null;
            }

            public CTreeNode LeftChild
            {
                get { return m_LeftChild; }
                set { m_LeftChild = value; }
            }

            public CTreeNode RightChild
            {
                get { return m_RightChild; }
                set { m_RightChild = value; }
            }
            public char Data
            {
                get { return m_Data; }
            }

            public int FindDepth()
            {
                int leftDepth = 0;
                int rightDepth = 0;

                if (null != m_LeftChild)
                    leftDepth = m_LeftChild.FindDepth();
                if (null != m_RightChild)
                    rightDepth = m_RightChild.FindDepth();

                return Math.Max(leftDepth, rightDepth) + 1;
            }

            public void Clone(CTreeNode newTreeNode)
            {
                if (null != this.m_LeftChild)
                {
                    newTreeNode.m_LeftChild = new CTreeNode(this.m_LeftChild.Data);
                    m_LeftChild.Clone(newTreeNode.m_LeftChild);
                }

                if (null != this.m_RightChild)
                {
                    newTreeNode.m_RightChild = new CTreeNode(this.m_RightChild.Data);
                    m_RightChild.Clone(newTreeNode.m_RightChild);
                }
            }

            public int InOrderPrintToArray(char[] array, int fillNext)
            {
                if (null != m_LeftChild)
                    fillNext = m_LeftChild.InOrderPrintToArray(array, fillNext);

                array[fillNext] = m_Data;
                fillNext++;

                if (null != m_RightChild)
                    fillNext = m_RightChild.InOrderPrintToArray(array, fillNext);
                return fillNext;
            }
            public void PrintStructure()
            {
                Console.Write( "Current data: {0}", m_Data );
                if (null != m_LeftChild)
                    Console.Write( " Left: {0} ", m_LeftChild.m_Data);
                else
                    Console.Write( " Left: NULL" );

                if (null != m_RightChild)
                    Console.Write(" Right: {0} ", m_RightChild.m_Data);
                else
                    Console.Write(" Right: NULL");
                Console.WriteLine();

                if (null != m_LeftChild)
                    m_LeftChild.PrintStructure();

                if (null != m_RightChild)
                    m_RightChild.PrintStructure();
            }
        }

        private CTreeNode m_Root;
        private int m_cNodes;

        public CBinarySearchTree()
        {
            m_Root = null;
        }

        public ErrorCode Add(char c)
        {
            ErrorCode code = ErrorCode.Success;
            if (null == m_Root)
            {
                m_cNodes++;
                m_Root = new CTreeNode(c);
            }
            else
            {
                CTreeNode cur = m_Root;
                while (true)
                {
                    if (cur.Data == c)
                        return ErrorCode.AleadyInTree;
                    else if (c < cur.Data)
                    {
                        if (cur.LeftChild == null)
                        {
                            cur.LeftChild = new CTreeNode(c);
                            m_cNodes++;
                            return ErrorCode.Success;
                        }
                        else
                            cur = cur.LeftChild;
                    }
                    else // ( c > cur.Data)
                    {
                        if (cur.RightChild == null)
                        {
                            cur.RightChild = new CTreeNode(c);
                            m_cNodes++;
                            return ErrorCode.Success;
                        }
                        else
                            cur = cur.RightChild;
                    }
                }
            }
            return code;
        }

        public bool Find(char c)
        {
            if (null == m_Root)
                return false;
            CTreeNode cur = m_Root;
            while (cur != null)
            {
                if (cur.Data == c)
                    return true;
                else if (c < cur.Data)
                    cur = cur.LeftChild;
                else // ( c > cur.Data)
                    cur = cur.RightChild;
            }
            return false;
        }

        public int FindDepth()
        {
            int depth = 0;
            if (null != m_Root)
            {
                depth = m_Root.FindDepth();
            }
            return depth;
        }

        public CBinarySearchTree Clone()
        {
            CBinarySearchTree newTree = new CBinarySearchTree();
            if (null != this.m_Root)
            {
                newTree.m_Root = new CTreeNode(this.m_Root.Data);
                this.m_Root.Clone(newTree.m_Root);
            }
            newTree.m_cNodes = this.m_cNodes;
            return newTree;
        }

        public char[] GetAllInOrder()
        {
            char[] array = new char[this.m_cNodes]; // m_cNodes may be 0
            if (null != m_Root)
                m_Root.InOrderPrintToArray(array, 0);

            return array;
        }

        public void PrintStructure()
        {
            if (null != m_Root)
                m_Root.PrintStructure();
            else
                Console.WriteLine("There are no values stored in the tree!");
        }

        public void Balance()
        {
            if (null == m_Root)
                return;

            char [] values = GetAllInOrder();
            int first = 0, last = values.Length - 1;

            m_Root = null;
            RecreateTree(first, last, values);
        }
        private void RecreateTree(int f, int l, char[] array)
        {
            int c = (f + l) / 2;

            Add(array[c]);

            if (c > f)
                RecreateTree(f, c - 1, array);

            if (l > c)
                RecreateTree(c + 1, l, array);
        }
    }
}