using System;
using System.Collections.Generic;
using System.Text;

namespace BSTSolution
{
    class ConsoleUI
    {
        public void RunProgram()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            bool fContinue = true;
            while (fContinue)
            {
                Console.WriteLine();
                Console.WriteLine();
                Console.WriteLine("Welcome to the console based, Binary Search Tree exercising program!");
                Console.WriteLine();
                Console.WriteLine("Your options at this point are:");
                Console.WriteLine();
                Console.WriteLine("\t0) Clear the tree (drop all values)");
                Console.WriteLine("\t1) Add a value to the tree");
                Console.WriteLine("\t2) Print all the values in the tree, in order\n\t\t(lowest value to highest value)");
                Console.WriteLine("\t3) Print out the structure of the tree\n\t\t(pre-order traversal)");
                Console.WriteLine("\t4) Print out the depth of the tree");
                Console.WriteLine("\t5) Search the tree, in an attempt to find a particular character");
                Console.WriteLine("\t6) Clone the tree");
                Console.WriteLine("\t7) Balance the tree");
                Console.WriteLine("\t8) Run the unit tests");
                Console.WriteLine("\t9) Exit this program");
                Console.WriteLine();
                Console.WriteLine("Type in the number corresponding to your choice, then press the 'Enter' key");
                Console.Write("> ");

                int choice;
                string str;
                CBinarySearchTree newBST;

                // NOTE: TryParse isn't available in the 2003 version - use Int32.Parse instead, and
                //          catch the exception if you want to handle the error of 'non-integer input'
                if (!Int32.TryParse(Console.ReadLine(), out choice))
                {
                    Console.WriteLine("You need to type in an integer");
                    continue;
                }
                if( choice < 0 || choice > 9)
                {
                    Console.WriteLine("You need to type in one of choices above.  You typed in: {0}", choice);
                    continue;
                }
                Console.Clear();

                switch (choice)
                {
                    case 0: // Ought to use enums or const int's or something :)
                        bst = new CBinarySearchTree();
                        break;
                    case 1: 
                        Console.WriteLine("What character would you like to add? (Type in the character, then press 'Enter'");
                        Console.Write("> ");
                        str = Console.ReadLine();
                        if (null == str || str.Length == 0)
                        {
                            Console.WriteLine("You need to type in a at least one character!");
                            continue;
                        }
                        CBinarySearchTree.ErrorCode ec = bst.Add(str[0]);
                        switch(ec )
                        {
                            case CBinarySearchTree.ErrorCode.AleadyInTree:
                                Console.WriteLine("I'm sorry, but {0} is already in the tree, and duplicate values are not allowed", str[0]);
                                continue;
                            case CBinarySearchTree.ErrorCode.Success:
                                Console.WriteLine("{0} has been successfully added to the tree", str[0]);
                                continue;
                        }
                        break;
                    case 2:
                        Console.WriteLine("Here are all the values in the tree, starting with the lowest/smallest, and going up to the highes/largest:");
                        char []contents = bst.GetAllInOrder();
                        for (int i = 0; i < contents.Length; i++)
                        {
                            if (i > 0)
                                Console.Write(", ");
                            Console.Write("{0}", contents[i]);
                        }
                        break;
                    case 3:
                        Console.WriteLine("Here are all the values in the tree:");
                        Console.WriteLine("\t(Pre-order traversal - 'structural' view");
                        bst.PrintStructure();
                        break;
                    case 4:
                        Console.WriteLine("The tree currently as a depth of: {0}", bst.FindDepth());
                        break;
                    case 5: 
                        Console.WriteLine("What character would you like to search for? (Type in the character, then press 'Enter'");
                        Console.Write("> ");
                        str = Console.ReadLine();
                        if (null == str || str.Length == 0)
                        {
                            Console.WriteLine("You need to type in a at least one character!");
                            continue;
                        }
                        Console.WriteLine("Searching for {0}", str[0]);
                        if (bst.Find(str[0]))
                        {
                            Console.WriteLine("We DID find {0} in the tree!", str[0]);
                        }
                        else
                        {
                            Console.WriteLine("We did NOT find {0} in the tree!", str[0]);
                        }
                        break;
                    case 6:
                        Console.WriteLine("Cloning the tree:");
                        Console.WriteLine("First: here is a 'Structural' (pre-order) view of the data in the ORIGINAL tree:");
                        Console.WriteLine();
                        bst.PrintStructure();
                        Console.WriteLine();
                        newBST = bst.Clone();
                        Console.WriteLine("Next: here is a 'Structural' (pre-order) view of the data in the CLONED tree:");
                        Console.WriteLine();
                        newBST.PrintStructure();
                        Console.WriteLine();
                        Console.WriteLine("Now discarding the CLONEd tree, and continuing to execute with the original:");
                        break;
                    case 7:
                        Console.WriteLine("Balancing the tree:");
                        Console.WriteLine("First: here is a 'Structural' (pre-order) view of the data in the tree, PRE-balance:");
                        Console.WriteLine();
                        bst.PrintStructure();
                        Console.WriteLine();
                        bst.Balance();
                        Console.WriteLine("Next: here is a 'Structural' (pre-order) view of the data in the tree, POST-balance:");
                        Console.WriteLine();
                        bst.PrintStructure();
                        Console.WriteLine();
                        Console.WriteLine("The program will now continue, using the newly balanced tree");
                        break;
                    case 8:
                        CBinarySearchTree_Tests cbst_t = new CBinarySearchTree_Tests();
                        cbst_t.RunAllTests();
                        break;
                    case 9:
                        fContinue = false;
                        break;
                } // end of switch
            } // end of main loop (while(fContinue))
        }
    }
}
