using System;
using System.Collections.Generic;
using System.Text;

namespace BSTSolution
{
    class CBinarySearchTree_Tests
    {
        public void RunAllTests()
        {
            Console.WriteLine("====================================================");
            Console.WriteLine("============ TESTS: ADDING TO THE TREE =============");
            Console.WriteLine("====================================================");
            AddOne();
            AddTwoL();
            AddTwoR();
            AddThreeB();
            AddThreeLL();
            AddThreeLR();
            AddThreeRL();
            AddThreeRR();
            AddABunch(0); // 0 means 'pick a new value'
            // if something goes wrong, note the seed value, then change
            // it here & recompile

            Console.WriteLine("====================================================");
            Console.WriteLine("============ TESTS: PRINTING THE TREE ==============");
            Console.WriteLine("====================================================");
            PrintOne();
            PrintTwoL();
            PrintTwoR();
            PrintThreeB();
            PrintThreeLL();
            PrintThreeLR();
            PrintThreeRL();
            PrintThreeRR();
            PrintABunch(0); // 0 means 'pick a new value'
            // if something goes wrong, note the seed value, then change
            // it here & recompile

            Console.WriteLine("====================================================");
            Console.WriteLine("====== TESTS: PRINTING THE DEPTH OF THE TREE =======");
            Console.WriteLine("====================================================");
            DepthOne();
            DepthTwo();
            DepthThree();
            DepthMore();


            Console.WriteLine("====================================================");
            Console.WriteLine("============ TESTS: SEARCHING THE TREE =============");
            Console.WriteLine("====================================================");
            FindEmpty();
            FindOne();
            FindTwoL();
            FindTwoR();
            FindThreeB();
            FindMore();

            Console.WriteLine("====================================================");
            Console.WriteLine("============== TESTS: CLONING THE TREE =============");
            Console.WriteLine("====================================================");

            CloneEmpty();
            CloneOne();
            CloneTwoL();
            CloneTwoR();
            CloneThreeB();
            CloneThreeLL();
            CloneThreeLR();
            CloneThreeRL();
            CloneThreeRR();


            Console.WriteLine("====================================================");
            Console.WriteLine("============ TESTS: BALANCING THE TREE =============");
            Console.WriteLine("====================================================");

            BalanceEmpty();
            BalanceOne();
            BalanceThreeR();
            BalanceThreeL();
            BalanceThreeLR();
            BalanceThreeRL();
            BalanceMore();
        }

        public void checkAddReturnValueSuccess(CBinarySearchTree.ErrorCode ec)
        {
            switch (ec)
            {
                case CBinarySearchTree.ErrorCode.AleadyInTree:
                    Console.WriteLine("**** TEST FAILED: Value already in tree ****");
                    break;
                case CBinarySearchTree.ErrorCode.Success:
                    break;
                default:
                    Console.WriteLine("**** ERROR: Return value of {0} not supported by tests! ****", ec);
                    break;
            }
        }

        // ADDING:
        public void AddOne()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            //            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" ADD A SINGLE ITEM TO AN EMPTY TREE");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("Current data: a Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('a'));

            bst.PrintStructure();
            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void AddTwoR()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" ADD TWO ITEMS (RIGHTWARD) TO AN EMPTY TREE");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("Current data: a Left: NULL Right: b");
            Console.WriteLine("Current data: b Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('a'));
            checkAddReturnValueSuccess(bst.Add('b'));
            bst.PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void AddTwoL()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" ADD TWO ITEMS (LEFTWARD) TO AN EMPTY TREE");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("Current data: b Left: a  Right: NULL");
            Console.WriteLine("Current data: a Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('b'));
            checkAddReturnValueSuccess(bst.Add('a'));
            bst.PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void AddThreeB()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" ADD THREE ITEMS (Balanced) TO AN EMPTY TREE");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("Current data: m Left: a  Right: t");
            Console.WriteLine("Current data: a Left: NULL Right: NULL");
            Console.WriteLine("Current data: t Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('m'));
            checkAddReturnValueSuccess(bst.Add('a'));
            checkAddReturnValueSuccess(bst.Add('t'));
            bst.PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void AddThreeLR()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" ADD THREE ITEMS (Left-Right) TO AN EMPTY TREE");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("Current data: m Left: a  Right: NULL");
            Console.WriteLine("Current data: a Left: NULL Right: b");
            Console.WriteLine("Current data: b Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('m'));
            checkAddReturnValueSuccess(bst.Add('a'));
            checkAddReturnValueSuccess(bst.Add('b'));
            bst.PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void AddThreeLL()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" ADD THREE ITEMS (Left-Left) TO AN EMPTY TREE");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("Current data: m Left: b  Right: NULL");
            Console.WriteLine("Current data: b Left: a  Right: NULL");
            Console.WriteLine("Current data: a Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('m'));
            checkAddReturnValueSuccess(bst.Add('b'));
            checkAddReturnValueSuccess(bst.Add('a'));
            bst.PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void AddThreeRL()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" ADD THREE ITEMS (Right-Left) TO AN EMPTY TREE");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("Current data: m Left: NULL Right: z");
            Console.WriteLine("Current data: z Left: t  Right: NULL");
            Console.WriteLine("Current data: t Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('m'));
            checkAddReturnValueSuccess(bst.Add('z'));
            checkAddReturnValueSuccess(bst.Add('t'));
            bst.PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void AddThreeRR()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" ADD THREE ITEMS (Right-Right) TO AN EMPTY TREE");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("Current data: m Left: NULL Right: t");
            Console.WriteLine("Current data: t Left: NULL Right: z");
            Console.WriteLine("Current data: z Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('m'));
            checkAddReturnValueSuccess(bst.Add('z'));
            checkAddReturnValueSuccess(bst.Add('t'));
            bst.PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void AddABunch(int seedValue)
        {
            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" ADD ~128 ITEMS (Random Order) TO AN EMPTY TREE");

            CBinarySearchTree bst = new CBinarySearchTree();
            Random rng;
            if (seedValue != 0)
                rng = new Random(seedValue);
            else
            {
                Console.WriteLine("Given 0 as the seed value - randomly generating a new value!");
                seedValue = (int)DateTime.Now.Ticks;
                rng = new Random(seedValue);
            }

            Console.WriteLine("Starting Seed Value: {0}", seedValue);

            Console.WriteLine();
            Console.WriteLine("You should see no errors, then the message 'Done!' ");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            char[] chars = new char[127];
            for (int i = 1; i < 128; i++)
            {
                chars[i - 1] = (char)i;
            }
            for (int i = 0; i < chars.Length; i++)
            {
                int iRandom = rng.Next(0, chars.Length);
                char temp = chars[iRandom];
                chars[iRandom] = chars[i];
                chars[i] = temp;
            }
            for (int i = 0; i < chars.Length; i++)
            {
                CBinarySearchTree.ErrorCode ec = bst.Add(chars[i]);
                checkAddReturnValueSuccess(ec);
            }
            Console.WriteLine();
            Console.WriteLine("Done!");
            Console.WriteLine("End Of Test:");
        }

        // PRINTING:
        private void PrintArray(char[] rg)
        {
            for (int i = 0; i < rg.Length; i++)
            {
                if (i > 0)
                    Console.Write(", ");

                Console.Write(rg[i]);
            }
        }

        public void PrintOne()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            //            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" PRINT A SINGLE ITEM (In-Order)");
            Console.WriteLine("You should see (via GetAllInOrder): ");
            Console.WriteLine("a");
            Console.WriteLine();
            //Console.WriteLine("Current data: a Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('a'));

            PrintArray(bst.GetAllInOrder());
            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void PrintTwoR()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" PRINT TWO ITEMS (RIGHTWARD)");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("a, b");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('a'));
            checkAddReturnValueSuccess(bst.Add('b'));

            PrintArray(bst.GetAllInOrder());
            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void PrintTwoL()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" PRINT TWO ITEMS (LEFTWARD)");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("a, b");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('b'));
            checkAddReturnValueSuccess(bst.Add('a'));

            PrintArray(bst.GetAllInOrder());
            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void PrintThreeB()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" PRINT THREE ITEMS (Balanced)");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("a, m, t");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('m'));
            checkAddReturnValueSuccess(bst.Add('a'));
            checkAddReturnValueSuccess(bst.Add('t'));

            PrintArray(bst.GetAllInOrder());
            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void PrintThreeLR()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" PRINT THREE ITEMS (Left-Right)");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("a, b, m");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('m'));
            checkAddReturnValueSuccess(bst.Add('a'));
            checkAddReturnValueSuccess(bst.Add('b'));

            PrintArray(bst.GetAllInOrder());
            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void PrintThreeLL()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" PRINT THREE ITEMS (Left-Left)");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("a, b, m");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('m'));
            checkAddReturnValueSuccess(bst.Add('b'));
            checkAddReturnValueSuccess(bst.Add('a'));

            PrintArray(bst.GetAllInOrder());
            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void PrintThreeRL()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" PRINT THREE ITEMS (Right-Left)");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("m, t, z");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('m'));
            checkAddReturnValueSuccess(bst.Add('z'));
            checkAddReturnValueSuccess(bst.Add('t'));

            PrintArray(bst.GetAllInOrder());
            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void PrintThreeRR()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" PRINT THREE ITEMS (Right-Right)");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("m, t, z");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('m'));
            checkAddReturnValueSuccess(bst.Add('z'));
            checkAddReturnValueSuccess(bst.Add('t'));

            PrintArray(bst.GetAllInOrder());
            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void PrintABunch(int seedValue)
        {
            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" ADD ~128 ITEMS (Random Order) TO AN EMPTY TREE, THEN PRINT");

            CBinarySearchTree bst = new CBinarySearchTree();
            Random rng;
            if (seedValue != 0)
                rng = new Random(seedValue);
            else
            {
                Console.WriteLine("Given 0 as the seed value - randomly generating a new value!");
                seedValue = (int)DateTime.Now.Ticks;
                rng = new Random(seedValue);
            }

            Console.WriteLine("Starting Seed Value: {0}", seedValue);

            Console.WriteLine();
            Console.WriteLine("You should see no errors, then 127 characters (in order), then the message 'Done!' ");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            char[] chars = new char[127];
            for (int i = 1; i < 128; i++)
            {
                chars[i - 1] = (char)i;
            }
            for (int i = 0; i < chars.Length; i++)
            {
                int iRandom = rng.Next(0, chars.Length);
                char temp = chars[iRandom];
                chars[iRandom] = chars[i];
                chars[i] = temp;
            }
            for (int i = 0; i < chars.Length; i++)
            {
                CBinarySearchTree.ErrorCode ec = bst.Add(chars[i]);
                checkAddReturnValueSuccess(ec);
            }

            PrintArray(bst.GetAllInOrder());
            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        // DEPTH:
        public void checkTreeDepth(CBinarySearchTree bst, int depth)
        {
            if (bst.FindDepth() != depth)
                Console.WriteLine("**** TEST FAILED: Tree supposed to be depth {0}, but is actually depth {1}****",
                    depth, bst.FindDepth());
        }

        public void DepthOne()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            //            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" THE DEPTH 1 TREE");
            Console.WriteLine("You should see: ");
            Console.WriteLine("Tree Depth: 1");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('a'));

            Console.WriteLine("Tree Depth: {0}", bst.FindDepth());
            checkTreeDepth(bst, 1);
            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void DepthTwo()
        {
            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" ALL DEPTH 2 TREES");
            Console.WriteLine("You should see: ");
            Console.WriteLine("Tree Depth (a,b): 2");
            Console.WriteLine("Tree Depth (b,a): 2");
            Console.WriteLine("Tree Depth (b,a,c): 2");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            CBinarySearchTree bst = new CBinarySearchTree();
            checkAddReturnValueSuccess(bst.Add('a'));
            checkAddReturnValueSuccess(bst.Add('b'));
            Console.WriteLine("Tree Depth (a,b): {0}", bst.FindDepth());
            checkTreeDepth(bst, 2);

            bst = new CBinarySearchTree();
            checkAddReturnValueSuccess(bst.Add('b'));
            checkAddReturnValueSuccess(bst.Add('a'));
            Console.WriteLine("Tree Depth (b,a): {0}", bst.FindDepth());
            checkTreeDepth(bst, 2);

            bst = new CBinarySearchTree();
            checkAddReturnValueSuccess(bst.Add('b'));
            checkAddReturnValueSuccess(bst.Add('a'));
            checkAddReturnValueSuccess(bst.Add('C'));
            Console.WriteLine("Tree Depth (b,a,c): {0}", bst.FindDepth());
            checkTreeDepth(bst, 2);

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void DepthThree()
        {
            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" A SAMPLING OF DEPTH 3 TREES");
            Console.WriteLine("You should see: ");
            Console.WriteLine("Tree Depth (c,b,a): 3");
            Console.WriteLine("Tree Depth (c,a,b): 3");
            Console.WriteLine("Tree Depth (d,b,a,c): 3");
            Console.WriteLine("Tree Depth (e,f,g): 3");
            Console.WriteLine("Tree Depth (e,g,f): 3");
            Console.WriteLine("Tree Depth (e,g,f,h): 3");
            Console.WriteLine("Tree Depth (e,g,f,h,a): 3");
            Console.WriteLine("Tree Depth (e,g,h,a): 3");
            Console.WriteLine("Tree Depth (e,g,h,a,b): 3");
            Console.WriteLine("Tree Depth (e,g,b,a): 3");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            // Left
            CBinarySearchTree bst = new CBinarySearchTree();
            checkAddReturnValueSuccess(bst.Add('c'));
            checkAddReturnValueSuccess(bst.Add('b'));
            checkAddReturnValueSuccess(bst.Add('a'));
            Console.WriteLine("Tree Depth (c,b,a): {0}", bst.FindDepth());
            checkTreeDepth(bst, 3);

            bst = new CBinarySearchTree();
            checkAddReturnValueSuccess(bst.Add('c'));
            checkAddReturnValueSuccess(bst.Add('a'));
            checkAddReturnValueSuccess(bst.Add('b'));
            Console.WriteLine("Tree Depth (c,a,b): {0}", bst.FindDepth());
            checkTreeDepth(bst, 3);

            bst = new CBinarySearchTree();
            checkAddReturnValueSuccess(bst.Add('d'));
            checkAddReturnValueSuccess(bst.Add('b'));
            checkAddReturnValueSuccess(bst.Add('a'));
            checkAddReturnValueSuccess(bst.Add('c'));
            Console.WriteLine("Tree Depth (d,b,a,c): {0}", bst.FindDepth());
            checkTreeDepth(bst, 3);

            // Right
            bst = new CBinarySearchTree();
            checkAddReturnValueSuccess(bst.Add('e'));
            checkAddReturnValueSuccess(bst.Add('f'));
            checkAddReturnValueSuccess(bst.Add('g'));
            Console.WriteLine("Tree Depth (e,f,g): {0}", bst.FindDepth());
            checkTreeDepth(bst, 3);

            bst = new CBinarySearchTree();
            checkAddReturnValueSuccess(bst.Add('e'));
            checkAddReturnValueSuccess(bst.Add('g'));
            checkAddReturnValueSuccess(bst.Add('f'));
            Console.WriteLine("Tree Depth (e,g,f): {0}", bst.FindDepth());
            checkTreeDepth(bst, 3);

            bst = new CBinarySearchTree();
            checkAddReturnValueSuccess(bst.Add('e'));
            checkAddReturnValueSuccess(bst.Add('g'));
            checkAddReturnValueSuccess(bst.Add('f'));
            checkAddReturnValueSuccess(bst.Add('h'));
            Console.WriteLine("Tree Depth (e,g,f,h): {0}", bst.FindDepth());
            checkTreeDepth(bst, 3);

            // Grab-bag:
            bst = new CBinarySearchTree();
            checkAddReturnValueSuccess(bst.Add('e'));
            checkAddReturnValueSuccess(bst.Add('g'));
            checkAddReturnValueSuccess(bst.Add('f'));
            checkAddReturnValueSuccess(bst.Add('h'));
            checkAddReturnValueSuccess(bst.Add('a'));
            Console.WriteLine("Tree Depth (e,g,f,h,a): {0}", bst.FindDepth());
            checkTreeDepth(bst, 3);

            bst = new CBinarySearchTree();
            checkAddReturnValueSuccess(bst.Add('e'));
            checkAddReturnValueSuccess(bst.Add('g'));
            checkAddReturnValueSuccess(bst.Add('h'));
            checkAddReturnValueSuccess(bst.Add('a'));
            checkAddReturnValueSuccess(bst.Add('b'));
            Console.WriteLine("Tree Depth (e,g,h,a,b): {0}", bst.FindDepth());
            checkTreeDepth(bst, 3);

            bst = new CBinarySearchTree();
            checkAddReturnValueSuccess(bst.Add('e'));
            checkAddReturnValueSuccess(bst.Add('g'));
            checkAddReturnValueSuccess(bst.Add('b'));
            checkAddReturnValueSuccess(bst.Add('a'));
            Console.WriteLine("Tree Depth (e,g,b,a): {0}", bst.FindDepth());
            checkTreeDepth(bst, 3);

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void DepthMore()
        {
            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" A SMALL SAMPLING OF DEPTH 4+ TREES");
            Console.WriteLine("You should see: ");
            Console.WriteLine("Tree Depth (d,c,b,a): 4");
            Console.WriteLine("Tree Depth (a,b,c,d): 4");
            Console.WriteLine("Tree Depth (a,b,c,d,e): 5");
            Console.WriteLine("Tree Depth (m,q,n,t,r,w,u,z): 5");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            CBinarySearchTree bst = new CBinarySearchTree();
            checkAddReturnValueSuccess(bst.Add('d'));
            checkAddReturnValueSuccess(bst.Add('c'));
            checkAddReturnValueSuccess(bst.Add('b'));
            checkAddReturnValueSuccess(bst.Add('a'));
            Console.WriteLine("Tree Depth (c,b,a): {0}", bst.FindDepth());
            checkTreeDepth(bst, 4);

            bst = new CBinarySearchTree();
            checkAddReturnValueSuccess(bst.Add('a'));
            checkAddReturnValueSuccess(bst.Add('b'));
            checkAddReturnValueSuccess(bst.Add('c'));
            checkAddReturnValueSuccess(bst.Add('d'));
            Console.WriteLine("Tree Depth (a,b,c,d): {0}", bst.FindDepth());
            checkTreeDepth(bst, 4);

            bst = new CBinarySearchTree();
            checkAddReturnValueSuccess(bst.Add('a'));
            checkAddReturnValueSuccess(bst.Add('b'));
            checkAddReturnValueSuccess(bst.Add('c'));
            checkAddReturnValueSuccess(bst.Add('d'));
            checkAddReturnValueSuccess(bst.Add('e'));
            Console.WriteLine("Tree Depth (a,b,c,d,e): {0}", bst.FindDepth());
            checkTreeDepth(bst, 5);

            bst = new CBinarySearchTree();
            checkAddReturnValueSuccess(bst.Add('m'));
            checkAddReturnValueSuccess(bst.Add('q'));
            checkAddReturnValueSuccess(bst.Add('n'));
            checkAddReturnValueSuccess(bst.Add('t'));
            checkAddReturnValueSuccess(bst.Add('r'));
            checkAddReturnValueSuccess(bst.Add('w'));
            checkAddReturnValueSuccess(bst.Add('u'));
            checkAddReturnValueSuccess(bst.Add('z'));
            Console.WriteLine("Tree Depth (m,q,n,t,r,w,u,z): {0}", bst.FindDepth());
            checkTreeDepth(bst, 5);


            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        // FINDING:
        private void printFindResult(CBinarySearchTree bst, char target)
        {
            if (bst.Find(target))
                Console.WriteLine("The value '{0}' IS present in the tree", target);
            else
                Console.WriteLine("The value '{0}' is NOT present in the tree", target);
        }

        public void FindEmpty()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            //            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" FIND IN AN EMPTY TREE");
            Console.WriteLine("You should see: ");
            Console.WriteLine();
            Console.WriteLine("The value 'a' is NOT present in the tree");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            printFindResult(bst, 'a');

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void FindOne()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" FIND IN A TREE WITH ONE ITEM");
            Console.WriteLine("You should see: ");
            Console.WriteLine();
            Console.WriteLine("The value 'b' IS present in the tree");
            Console.WriteLine("The value 'a' is NOT present in the tree");
            Console.WriteLine("The value 'c' is NOT present in the tree");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('b'));
            printFindResult(bst, 'b');
            printFindResult(bst, 'a');
            printFindResult(bst, 'c');

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void FindTwoL()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" FIND IN A TREE WITH TWO ITEMS (Leftward)");
            Console.WriteLine("You should see: ");
            Console.WriteLine();
            Console.WriteLine("The value 'm' IS present in the tree");
            Console.WriteLine("The value 'g' IS present in the tree");
            Console.WriteLine("The value 'a' is NOT present in the tree");
            Console.WriteLine("The value 'h' is NOT present in the tree");
            Console.WriteLine("The value 'z' is NOT present in the tree");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('m'));
            checkAddReturnValueSuccess(bst.Add('g'));
            printFindResult(bst, 'm');
            printFindResult(bst, 'g');
            printFindResult(bst, 'a');
            printFindResult(bst, 'h');
            printFindResult(bst, 'z');

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void FindTwoR()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" FIND IN A TREE WITH TWO ITEMS (Rightward)");
            Console.WriteLine("You should see: ");
            Console.WriteLine();
            Console.WriteLine("The value 'm' IS present in the tree");
            Console.WriteLine("The value 't' IS present in the tree");
            Console.WriteLine("The value 'a' is NOT present in the tree");
            Console.WriteLine("The value 'n' is NOT present in the tree");
            Console.WriteLine("The value 'z' is NOT present in the tree");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('m'));
            checkAddReturnValueSuccess(bst.Add('t'));
            printFindResult(bst, 'm');
            printFindResult(bst, 't');
            printFindResult(bst, 'a');
            printFindResult(bst, 'n');
            printFindResult(bst, 'z');

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }


        public void FindThreeB()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" FIND IN A TREE WITH THREE ITEMS (Balanced)");
            Console.WriteLine("You should see: ");
            Console.WriteLine();
            Console.WriteLine("The value 'm' IS present in the tree");
            Console.WriteLine("The value 't' IS present in the tree");
            Console.WriteLine("The value 'g' IS present in the tree");
            Console.WriteLine("The value 'a' is NOT present in the tree");
            Console.WriteLine("The value 'h' is NOT present in the tree");
            Console.WriteLine("The value 'o' is NOT present in the tree");
            Console.WriteLine("The value 'z' is NOT present in the tree");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('m'));
            checkAddReturnValueSuccess(bst.Add('t'));
            checkAddReturnValueSuccess(bst.Add('g'));
            printFindResult(bst, 'm');
            printFindResult(bst, 't');
            printFindResult(bst, 't');
            printFindResult(bst, 'a');
            printFindResult(bst, 'h');
            printFindResult(bst, 'o');
            printFindResult(bst, 'z');

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void FindMore()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" FIND IN A LARGER TREE");
            Console.WriteLine("You should see: ");
            Console.WriteLine();
            Console.WriteLine("The value 'm' IS present in the tree");
            Console.WriteLine("The value 't' IS present in the tree");
            Console.WriteLine("The value 'n' IS present in the tree");
            Console.WriteLine("The value 'o' IS present in the tree");
            Console.WriteLine("The value 'g' IS present in the tree");
            Console.WriteLine("The value 'b' IS present in the tree");
            Console.WriteLine("The value 'c' IS present in the tree");
            Console.WriteLine("The value 'c' IS present in the tree");
            Console.WriteLine("The value 'd' is NOT present in the tree");
            Console.WriteLine("The value 'f' is NOT present in the tree");
            Console.WriteLine("The value 'l' is NOT present in the tree");
            Console.WriteLine("The value 'q' is NOT present in the tree");
            Console.WriteLine("The value 'z' is NOT present in the tree");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('m'));
            checkAddReturnValueSuccess(bst.Add('t'));
            checkAddReturnValueSuccess(bst.Add('n'));
            checkAddReturnValueSuccess(bst.Add('o'));
            checkAddReturnValueSuccess(bst.Add('g'));
            checkAddReturnValueSuccess(bst.Add('b'));
            checkAddReturnValueSuccess(bst.Add('a'));
            checkAddReturnValueSuccess(bst.Add('c'));
            printFindResult(bst, 'm');
            printFindResult(bst, 't');
            printFindResult(bst, 'n');
            printFindResult(bst, 'o');
            printFindResult(bst, 'g');
            printFindResult(bst, 'b');
            printFindResult(bst, 'a');
            printFindResult(bst, 'c');

            printFindResult(bst, 'd');
            printFindResult(bst, 'f');
            printFindResult(bst, 'l');
            printFindResult(bst, 'q');
            printFindResult(bst, 'z');

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        //CLONING:
        public void CloneEmpty()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            //            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" CLONE AN EMPTY TREE");
            Console.WriteLine("You should see something SIMILAR to (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("<There is no data in the tree>");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            bst.Clone().PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void CloneOne()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            //            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" CLONE A TREE CONTAINING A SINGLE ITEM");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("Current data: a Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('a'));
            bst.Clone().PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void CloneTwoR()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" CLONE A TREE CONTAINING TWO ITEMS (Right-ward)");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("Current data: a Left: NULL Right: b");
            Console.WriteLine("Current data: b Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('a'));
            checkAddReturnValueSuccess(bst.Add('b'));
            bst.Clone().PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void CloneTwoL()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" CLONE A TREE CONTAINING TWO ITEMS (Left-ward)");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("Current data: b Left: a  Right: NULL");
            Console.WriteLine("Current data: a Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('b'));
            checkAddReturnValueSuccess(bst.Add('a'));
            bst.Clone().PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void CloneThreeB()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" CLONE A TREW WITH THREE ITEMS (Balanced)");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("Current data: m Left: a  Right: t");
            Console.WriteLine("Current data: a Left: NULL Right: NULL");
            Console.WriteLine("Current data: t Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('m'));
            checkAddReturnValueSuccess(bst.Add('a'));
            checkAddReturnValueSuccess(bst.Add('t'));
            bst.Clone().PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void CloneThreeLR()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" CLONE A TREE WITH THREE ITEMS (Left-Right)");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("Current data: m Left: a  Right: NULL");
            Console.WriteLine("Current data: a Left: NULL Right: b");
            Console.WriteLine("Current data: b Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('m'));
            checkAddReturnValueSuccess(bst.Add('a'));
            checkAddReturnValueSuccess(bst.Add('b'));
            bst.Clone().PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void CloneThreeLL()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" CLONE A TREE WITH 3 ITEMS (Left-Left)");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("Current data: m Left: b  Right: NULL");
            Console.WriteLine("Current data: b Left: a  Right: NULL");
            Console.WriteLine("Current data: a Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('m'));
            checkAddReturnValueSuccess(bst.Add('b'));
            checkAddReturnValueSuccess(bst.Add('a'));
            bst.Clone().PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void CloneThreeRL()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" CLONE A TREE WITH THREE ITEMS (Right-Left)");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("Current data: m Left: NULL Right: z");
            Console.WriteLine("Current data: z Left: t  Right: NULL");
            Console.WriteLine("Current data: t Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('m'));
            checkAddReturnValueSuccess(bst.Add('z'));
            checkAddReturnValueSuccess(bst.Add('t'));
            bst.Clone().PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void CloneThreeRR()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" CLONE A TREE WITH THREE ITEMS (Right-Right)");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("Current data: m Left: NULL Right: t");
            Console.WriteLine("Current data: t Left: NULL Right: z");
            Console.WriteLine("Current data: z Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('m'));
            checkAddReturnValueSuccess(bst.Add('t'));
            checkAddReturnValueSuccess(bst.Add('z'));
            bst.Clone().PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        //BALANCING:
        public void BalanceEmpty()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            //            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" BALANCING AN EMPTY TREE");
            Console.WriteLine("You should see something SIMILAR to (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("<There is no data in the tree>");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            bst.Balance();
            bst.PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void BalanceOne()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" BALANCE A TREE CONTAINING A SINGLE ITEM");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("Current data: a Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('a'));
            bst.Balance();
            bst.PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void BalanceThreeR()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" BALANCE A TREE CONTAINING THREE  ITEMS (RR)");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("Current data: b Left: a Right: c");
            Console.WriteLine("Current data: a Left: NULL Right: NULL");
            Console.WriteLine("Current data: c Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('a'));
            checkAddReturnValueSuccess(bst.Add('b'));
            checkAddReturnValueSuccess(bst.Add('c'));
            bst.Balance();
            bst.PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void BalanceThreeL()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" BALANCE A TREE CONTAINING THREE  ITEMS (LL)");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("Current data: b Left: a Right: c");
            Console.WriteLine("Current data: a Left: NULL Right: NULL");
            Console.WriteLine("Current data: c Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('c'));
            checkAddReturnValueSuccess(bst.Add('b'));
            checkAddReturnValueSuccess(bst.Add('a'));
            bst.Balance();
            bst.PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void BalanceThreeLR()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" BALANCE A TREE CONTAINING THREE  ITEMS (LR)");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("Current data: b Left: a Right: c");
            Console.WriteLine("Current data: a Left: NULL Right: NULL");
            Console.WriteLine("Current data: c Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('b'));
            checkAddReturnValueSuccess(bst.Add('a'));
            checkAddReturnValueSuccess(bst.Add('c'));
            bst.Balance();
            bst.PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void BalanceThreeRL()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" BALANCE A TREE CONTAINING THREE ITEMS (RL)");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("Current data: b Left: a Right: c");
            Console.WriteLine("Current data: a Left: NULL Right: NULL");
            Console.WriteLine("Current data: c Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('a'));
            checkAddReturnValueSuccess(bst.Add('c'));
            checkAddReturnValueSuccess(bst.Add('b'));
            bst.Balance();
            bst.PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }

        public void BalanceMore()
        {
            CBinarySearchTree bst = new CBinarySearchTree();

            Console.WriteLine("================================");
            Console.WriteLine();
            Console.WriteLine(" BALANCE A TREE CONTAINING TWO ITEMS (Right-ward)");
            Console.WriteLine("You should see (via PrintStructure): ");
            Console.WriteLine();
            Console.WriteLine("Current data: h Left: d  Right: l");
            Console.WriteLine("Current data: d Left: b  Right: f");
            Console.WriteLine("Current data: b Left: a  Right: c");
            Console.WriteLine("Current data: a Left: NULL Right: NULL");
            Console.WriteLine("Current data: c Left: NULL Right: NULL");
            Console.WriteLine("Current data: f Left: e  Right: g");
            Console.WriteLine("Current data: e Left: NULL Right: NULL");
            Console.WriteLine("Current data: g Left: NULL Right: NULL");
            Console.WriteLine("Current data: l Left: j  Right: n");
            Console.WriteLine("Current data: j Left: i  Right: k");
            Console.WriteLine("Current data: i Left: NULL Right: NULL");
            Console.WriteLine("Current data: k Left: NULL Right: NULL");
            Console.WriteLine("Current data: n Left: m  Right: o");
            Console.WriteLine("Current data: m Left: NULL Right: NULL");
            Console.WriteLine("Current data: o Left: NULL Right: NULL");
            Console.WriteLine();
            Console.WriteLine("Start Of Test:");
            Console.WriteLine();

            checkAddReturnValueSuccess(bst.Add('a'));
            checkAddReturnValueSuccess(bst.Add('b'));
            checkAddReturnValueSuccess(bst.Add('c'));
            checkAddReturnValueSuccess(bst.Add('d'));
            checkAddReturnValueSuccess(bst.Add('e'));
            checkAddReturnValueSuccess(bst.Add('f'));
            checkAddReturnValueSuccess(bst.Add('g'));
            checkAddReturnValueSuccess(bst.Add('h'));
            checkAddReturnValueSuccess(bst.Add('i'));
            checkAddReturnValueSuccess(bst.Add('j'));
            checkAddReturnValueSuccess(bst.Add('k'));
            checkAddReturnValueSuccess(bst.Add('l'));
            checkAddReturnValueSuccess(bst.Add('m'));
            checkAddReturnValueSuccess(bst.Add('n'));
            checkAddReturnValueSuccess(bst.Add('o'));
            bst.Balance();
            bst.PrintStructure();

            Console.WriteLine();
            Console.WriteLine("End Of Test:");
        }
    }
}
