using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

// Our own library
using XNACS1Lib;

namespace AlphabetHunt
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public abstract class CTreasureBox
    {
        protected const float kBoxWidth = 1.2f;
        protected const float kHalfBoxWidth = 0.5f * kBoxWidth;

        protected XNACS1Rectangle m_TreasureObject;
        protected int m_AlphaTexIndex;

        static protected float m_WorldMax;
        static public void SetWorldMax(float m) { m_WorldMax = m; }

        static public char[] kCharArray = {
            'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 
            'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 
            'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z' };


        // Subclass should initialize these
        protected const float kBitMapRatio = 235.0f / 295.0f; // width to height ratio
        protected const float kAlphaFraction = 0.6f;
        protected const float kAlphaWidth = kAlphaFraction * kBoxWidth;
        protected const float kAlphaHeight = kAlphaWidth * (1.0f/kBitMapRatio);
        protected XNACS1Rectangle m_AlphaBox;    
        protected String m_AlphaTexture = null;
        protected bool m_HasAlpha;

        public CTreasureBox(float xPos)
        {
            m_TreasureObject = new XNACS1Rectangle();
            m_AlphaBox = new XNACS1Rectangle();
         
            m_TreasureObject.CenterX= xPos;
            m_TreasureObject.CenterY = 0.0f;
            m_AlphaTexIndex = (int)(CTreasureEngine.RandomNumber() * 26.0f);
            m_HasAlpha = true;
        }

        public void UpdateTreasureBox(CSideScroll scroll)
        {
            m_TreasureObject.CenterX = scroll.UpdatePosition(m_TreasureObject.CenterX);
            m_AlphaBox.CenterX = scroll.UpdatePosition(m_AlphaBox.CenterX);
        }

        public bool OutsideTheWorld()
        {
            return (XNACS1Base.World.OutsideWorldBound(m_TreasureObject));
        }

        protected abstract void TreasureScores(CScoreSystem score);

        public void CollectTreasure(CScoreSystem score, CAlphabetHero hero)
        {
            if (m_HasAlpha && AlphaBox().Collided(hero.GetPrimitive()))
            {
                TreasureScores(score);
                m_HasAlpha = false;
                AlphaBox().RemoveFromAutoDrawSet();
            }
        }

        protected abstract XNACS1Primitive AlphaBox();

        public void DestroyTresure()
        {
            if (m_HasAlpha)
                AlphaBox().RemoveFromAutoDrawSet();
            m_TreasureObject.RemoveFromAutoDrawSet();
        }
    }
}