using System;
using System.Collections.Generic;

namespace AlphabetHunt
{
    public class CBinarySearchTree : CBinarySearchTreeBase
    {
        public enum ErrorCode
        {
            Success = 0,
            AleadyInTree = 1
        }

        private class CTreeNode : CTreeNodeBase
        {
            private CTreeNode m_LeftChild, m_RightChild;
            private char m_Data;

            public CTreeNode(char data)
            {
                m_Data = data;
                m_LeftChild = null;
                m_RightChild = null;
            }

            public override char Data
            {
                get { return m_Data; }
                set { m_Data = value; }
            }
            public override CTreeNodeBase LeftChild
            {
                get { return m_LeftChild; }
				set { m_LeftChild = (CTreeNode) value; }
            }
            public override CTreeNodeBase RightChild
            {
                get { return m_RightChild;}
				set { m_RightChild = (CTreeNode) value; }
            }

            public ErrorCode Add(char c)
            {
                ErrorCode code = ErrorCode.Success;

                if (c == m_Data)
                {
                    code = ErrorCode.AleadyInTree;
                }
                else
                {
                    if (c < m_Data)
                        if (null != m_LeftChild)
                            code = m_LeftChild.Add(c);
                        else
                            m_LeftChild = new CTreeNode(c);
                    else
                        if (null != m_RightChild)
                            code = m_RightChild.Add(c);
                        else
                            m_RightChild = new CTreeNode(c);
                }
                return code;
            }

            public bool Find(char c)
            {
                bool found = (c == m_Data);
                if ((!found) && (null != m_LeftChild))
                    found = m_LeftChild.Find(c);
                if ((!found) && (null != m_RightChild))
                    found = m_RightChild.Find(c);
                return found;
            }

            public void InOrderPrintToList(List<char> list)
            {
                if (null != m_LeftChild)
                    m_LeftChild.InOrderPrintToList(list);

                list.Add(m_Data);

                if (null != m_RightChild)
                    m_RightChild.InOrderPrintToList(list);
            }

            public void InOrderPrintToArray(char [] result, ref int n)
            {
                if (null != m_LeftChild)
                    m_LeftChild.InOrderPrintToArray(result, ref n);

                result[n] = m_Data;
                n++;

                if (null != m_RightChild)
                    m_RightChild.InOrderPrintToArray(result, ref n);
            }

            public int FindDepth(int depth)
            {
                depth = depth + 1;
                int leftDepth = depth, rightDepth = depth;

                if (null != m_LeftChild)
                    leftDepth = m_LeftChild.FindDepth(depth);

                if (null != m_RightChild)
                    rightDepth = m_RightChild.FindDepth(depth);

                return (leftDepth > rightDepth) ? leftDepth : rightDepth;
            }

            public void Clone(CTreeNode newNode)
            {
                if (null != this.m_LeftChild) 
                {
                    newNode.m_LeftChild = new CTreeNode( this.m_LeftChild.Data );
                    m_LeftChild.Clone(newNode.m_LeftChild);
                }

                if (null != this.m_RightChild)
                {
                    newNode.m_RightChild = new CTreeNode(this.m_RightChild.Data);
                    m_RightChild.Clone(newNode.m_RightChild);
                }
            }
        }

        private CTreeNode m_Root;
        private int m_cNodes;

        protected override CTreeNodeBase Root { get { return m_Root; } }

        public CBinarySearchTree()
        {
            m_Root = null;
            m_cNodes = 0;
        }


        public ErrorCode Add(char c)
        {
            ErrorCode code = ErrorCode.Success;
            if (null == m_Root)
            {
                m_cNodes = 1;
                m_Root = new CTreeNode(c);
            }
            else
            {
                code = m_Root.Add(c);
                if (ErrorCode.Success == code)
                    m_cNodes++;
            }
            return code;
        }

        public bool Find(char c)
        {
            bool found = false;
            if (null != m_Root)
                found = m_Root.Find(c);
            return found;
        }

        public char[] GetAllInOrder()
        {
            char[] result = new char[m_cNodes];
            int num = 0;
            if (null != m_Root)
                m_Root.InOrderPrintToArray(result, ref num);
            return result;
        }

        public int FindDepth()
        {
            int depth = 0;
            if (null != m_Root)
            {
                depth = m_Root.FindDepth(-1);
            }
            return depth;
        }

        public CBinarySearchTree Clone()
        {
            CBinarySearchTree newTree = new CBinarySearchTree();
            if (null != this.m_Root)
            {
                newTree.m_Root = new CTreeNode(this.m_Root.Data);
                m_Root.Clone(newTree.m_Root);
            }
            newTree.m_cNodes = this.m_cNodes;
            return newTree;
        }

        private void RecreateTree(int f, int l, char[] rgValues)
        {
            int c = (f + l) / 2;

            Add(rgValues[c]);

            if (c > f)
                RecreateTree(f, c - 1, rgValues);

            if (l > c)
                RecreateTree(c + 1, l, rgValues);
        }

        public void Balance()
        {
            if (null == m_Root)
                return;

            // char[] rgValues = GetAllInOrder().ToArray();
            char[] rgValues = GetAllInOrder();
            int first = 0, last = rgValues.Length - 1;

            m_Root = null;
            RecreateTree(first, last, rgValues);
        }

    }
}
