using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

using XNACS1Lib;

namespace AlphabetHunt
{
    public abstract class CBinarySearchTreeBase
    {
        abstract protected CTreeNodeBase Root { get; }

        protected abstract class CTreeNodeBase
        {
            abstract public char Data { get; set; }
            abstract public CTreeNodeBase LeftChild { get; set;  }
            abstract public CTreeNodeBase RightChild { get; set; }

            private Vector2 DrawChild(XNACS1PrimitiveSet tree, CTreeNodeBase child, Vector2 myPos, float width, float xOffset, float yOffset)
            {
                Vector2 cPos = new Vector2();
                cPos.X = myPos.X + (1.5f*xOffset);
                cPos.Y = myPos.Y + yOffset;
                Vector2 center = (myPos + cPos) / 2.0f;
                XNACS1Rectangle r = new XNACS1Rectangle(myPos, cPos, 0.2f * width, null);
                r.OutsideColor = r.CenterColor = Color.White;
                tree.AddToSet(r);

                cPos.X += xOffset;
                cPos.Y += yOffset;
                return cPos;
            }

            public void InOrderDraw(XNACS1PrimitiveSet tree, Vector2 myPos, float space, float width)
            {
                if (null != LeftChild)
                {
                    Vector2 left = DrawChild(tree, LeftChild, myPos, width, -(2.0f*space), -width);
                    LeftChild.InOrderDraw(tree, left, space*0.55f, width);
                }
                if (null != RightChild)
                {
                    Vector2 right = DrawChild(tree, RightChild, myPos, width, (2.0f*space), -width);
                    RightChild.InOrderDraw(tree, right, space*0.55f, width);
                }   
                String useTexture = String.Concat(Data);
                XNACS1Rectangle r = new XNACS1Rectangle(myPos, width, width, useTexture);
                tree.AddToSet(r);
            }
        }

        public void InOrderDraw(Vector2 pos, float width, XNACS1PrimitiveSet tree)
        {
            if (null != Root)
            {
                tree.RemoveAllFromSet();
                Root.InOrderDraw(tree, pos, 1.5f * width, width); // defined in the CTreeNodeBase class.
            }
        }
    }
}
