using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

// Our own library
using XNACS1Lib;

namespace AlphabetHunt
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class CTreasureEngine
    {
        public const float kFloorY = 1.5f;
        public static Random m_Rand = new Random();
        public static float RandomNumber() { return (float) m_Rand.NextDouble(); }

        private List<CTreasureBox> m_Treasures;
        private int m_UpdatesSinceLastCreate;
        
        private Vector2 m_WorldSize;
        private CSideScroll m_BackgroundScroll;

        public CTreasureEngine(Vector2 worldSize, CSideScroll bg)
        {
            m_WorldSize = worldSize;
            m_Treasures = new List<CTreasureBox>();

            m_BackgroundScroll = bg;

            m_UpdatesSinceLastCreate = 0;
        }

        public void UpdateTreasureEngine(CAlphabetHero hero, CScoreSystem score)
        {
            int num = m_Treasures.Count;
            
            m_UpdatesSinceLastCreate++;

            for (int i = num-1; i>=0 ; i--) {
                CTreasureBox t = m_Treasures[i];

                t.UpdateTreasureBox(m_BackgroundScroll);
                t.CollectTreasure(score, hero);

                if (t.OutsideTheWorld())
                {
                    t.DestroyTresure();
                    m_Treasures.RemoveAt(i);
                }
                    
            }

            float choice = CTreasureEngine.RandomNumber();
            if ((m_UpdatesSinceLastCreate > 50) && (choice > 0.95f))
            {
                float x = m_WorldSize.X;
                CTreasureBox t = new CRoadTreasureBox(x);
                m_Treasures.Add(t);
                m_UpdatesSinceLastCreate = 0;
            }
        }
    }
}