using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

// Our own library
using XNACS1Lib;

namespace AlphabetHunt
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class AlphabetHunt : XNACS1Base
    {
        private const float WORLD_MIN_X = 0.0f;
        private const float WORLD_MIN_Y = 0.0f;
        private const float WORLD_WIDTH = 18.0f;    // with ratio of 1280/720, Width of 18 means, height is about 10 

        private CSideScroll m_BgScroll;
        private CAlphabetHero m_Hero;
        private CTreasureEngine m_TreasureEngine;
        private CScoreSystem m_ScoreSystem;
        private XNACS1PrimitiveSet m_CharListSet;

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void InitializeWorld()
        {
            World.SetWorldCoordinate(new Vector2(WORLD_MIN_X, WORLD_MIN_Y), WORLD_WIDTH);

            m_BgScroll = new CSideScroll(@"bg", 1700, World.WorldMax);
            m_TreasureEngine = new CTreasureEngine(World.WorldMax, m_BgScroll);
            m_ScoreSystem = new CScoreSystem();
          
            m_CharListSet = new XNACS1PrimitiveSet();

            m_Hero = new CAlphabetHero(World.WorldMax.X, World.WorldMax.Y, m_BgScroll);
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        protected override void UpdateWorld()
        {
            if (ButtonState.Pressed == GamePad.Buttons.Back)
                Exit();

            if (GamePad.ButtonStartClicked())
                World.Paused = !World.Paused;

            if (GamePad.ButtonBClicked())
            {
                m_ScoreSystem.ResetScores();
                XNACS1Base.PlayACue( "button");
            }

            if (!World.Paused)
            {
                m_ScoreSystem.ClearStatus();

                if (GamePad.ButtonAClicked())
                {
                    m_Hero.HeroJump();
                    XNACS1Base.PlayACue( "fly");
                }

                m_BgScroll.UpdateScroll();

                m_Hero.SetHeroVelocity(GamePad.ThumbSticks.Left);
                m_Hero.UpdateHero();

                m_TreasureEngine.UpdateTreasureEngine(m_Hero, m_ScoreSystem);
            }

            EchoToBottomStatus("Total Score: " + m_ScoreSystem.GetScore());

            String status = m_ScoreSystem.GetStatus();

            if (null != status)
                EchoToTopStatus(status);
                
            FillCharList(1.0f, m_ScoreSystem.InsertionOrder, m_CharListSet);
        }

        private void FillCharList(float pos, List<char> list, XNACS1PrimitiveSet drawSet)
        {
            drawSet.RemoveAllFromSet();

            float width = World.WorldMax.X / 39.0f;
            Vector2 p = new Vector2();
            p.X = width * 0.5f;
            p.Y = pos;
            for (int i = 0; i < list.Count; i++)
            {
                String t = String.Concat(list[i]);
                XNACS1Rectangle r = new XNACS1Rectangle(p, width, width, t);
                drawSet.AddToSet(r);
                p.X += (1.5f * width);
            }
        }        
    }
}
