
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

// Our own library
using XNACS1Lib;


namespace SimpleXNA
{
    class Block
    {
        protected float m_BlockWidth = 2.0f;
        protected float m_BlockHeight = 1.0f;

        #region Instance variable: a rectangular block
        protected XNACS1Rectangle m_Rec; // the block
        #endregion

        #region Constructor
        public Block(float x, float y)
        {
            m_Rec = new XNACS1Rectangle(new Vector2(x, y), m_BlockWidth, m_BlockHeight, "blockTexture");
        }
        #endregion

        #region Get block position and bounds
        public Vector2 Position {
            set { m_Rec.Center = value; }
            get { return m_Rec.Center; } 
        }
        #endregion

        #region Collide With Ball!
        public bool CollideWithBall(Ball b)
        {
            XNACS1Circle bc = b.GetCircle;

            if (!m_Rec.Collided(bc))
                return false;

            Vector2 v = b.Velocity;

            b.Bounce();
            if (bc.LeftOf(m_Rec) && (v.X > 0))
            {     // moving right-ward into the left side
                v.X = -v.X;
            }
            else if ((bc.RightOf(m_Rec)) && (v.X < 0))
            {   // moving left-ward into the right side
                v.X = -v.X;
            }
            else
            {
                v.Y = -v.Y;
            }
            b.Velocity = v;

            return true;
        }
        #endregion

        public void DeleteBlock()
        {
            m_Rec.RemoveFromAutoDrawSet();
        }
    }
}